
# module for all accessing the database in which all data is stored
# all operations involving inserts, updates and deletes are performed
# in this module

namespace eval DBOperations {
	variable db_is_open 0
	variable reverse_lookup 
	variable reverse_lookup_exists 0

	source [file join "$global_options(BASEDIR)" dbinfo.tcl]
	source [file join "$global_options(BASEDIR)" tableinfo.tcl]
	
	# copy an element of a questionary to another position
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					orderno: unique identifier of the element to be copied
	#					neworderno: unique id of the new element
	namespace export el_Copy
	
	# delete an element of a questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					orderno: unique identifier of the element to be deleted
	namespace export el_Delete
	
	# edit an element of a questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					orderno: unique identifier of the element to be edited
	#					type: kind of the element to be changed
	#					parameterlist: new data describing the changed element
	namespace export el_Edit
	
	# insert a new element into a questionary at any position
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					orderno: unique identifier of the element to be inserted
	#					type: kind of the element to be inserted
	#					parameterlist: data describing the new element
	namespace export el_Insert
	
	# append a new element at the end of a questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					type: kind of the new element
	#					parameterlist: data describing the new element
	namespace export el_Append
	
	# change the postion of an element in a questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					orderno: unique identifier of the element to be moved, also the position
	#					neworderno: new position the element will be moved to 
	namespace export el_Move
	
	# copy a complete questionary
	# parameters:	sourceformid: unique identifier of the questionary to be copied
	#					title: title of the new questionary
	#					password: authentification to copy the questionary
	#					passwordnew: password for the new questionary
	#					passwordcheck: to confirm the password for the new questionary
	namespace export fbg_Copy
	
	# delete a complete questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to delete the questionary
	namespace export fbg_Delete
	
	# change the general data of a questionary
	# parameters:	formid: unique identifier of the questionary
	#					password: authentification to change the questionary
	#					title: new title of the questionary
	#					passwordnew: new password for the questionary
	#					passwordcheck: confirmation of the new password
	#					parameterlist: data describing the general appearance of a questionary
	namespace export fbg_Edit

	# generate a questionary from the database to be filled out by a user
	# parameters:	formid: unique identifier of the questionary
	namespace export fbg_Generate

	# create a new questionary 	
	# parameters:	title: title of the new questionary
	#					password: password for the new questionary, for authentification purposes
	#					passwordcheck: confirmation of the new password
	#					parameterlist: data describing the general appearance of the new questionary
	namespace export fbg_New
	
	# change the title of an existing questionary
	# parameters:	formid: unique identifier of the questionary
	#					title: new title of the questionary
	#					password: authentification to change the questionary
	namespace export fbg_Rename
	
	# save a filled out and commited questionary to the results table
	# parameters:	formid: unique identifier of the committed questionary
	#					answers: list of answers
	namespace export fbg_Result
	
	# builds a list of all forms stored in the database
	# parameters:
	namespace export build_Form_List
	
	# builds a list of all elements belonging to a questionary
	# parameters:	formid: unique identifier of the requested questionary
	namespace export build_Element_List
	
	# opens a connection to the database
	# parameters:
	namespace export db_Is_Open
	
	# closes the connection to the database
	# parameters:
	namespace export close_DB
	
	# extracts a questionary's title from the database
	# parameters:	formid: unique identifier of the requested questionary
	namespace export get_Form_Title
	
	# checks if a given password matches the one saved in the database
	# parameters:	formid: unique identifier of the questionary requested
	#					password: the password to be checked
	namespace export password_Matches
	
	# checks if there are any visible elements in the questionary
	# parameters:	formid: unique identifier of the questionary to be checked
	namespace export no_Visible_Elements
	
	# checks if a form with the given formid exists
	# parameters:	formid: unique identifier to be checked for
	namespace export form_Exists
	
	# checks the type of the element specified
	# parameters:	formid: unique identifier of a questionary
	#					orderno: unique identifier of a questionary's element
	namespace export get_Element_Type
	
	# selects an element by its orderno
	# parameters:	formid: unique identifier of a questionary
	#					orderno: unique identifier of a questionary's element
	namespace export select_OrderNo
}	



########################################################################
###########################    Oeffentlich    ##########################
########################################################################


proc DBOperations::el_Copy	{	{formid {}}
										{password {}}
										{orderno {}}
										{neworderno {}}
										{elementtype {}}
										{parameterlist {}}
									}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# check the orderno of the element to be copied, which must exist
	set checkresult [Util::check_OrderNo "$formid" "$orderno" "hastoexist"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# determine the position at which the element will be inserted into
	# the questionary 
	if {$neworderno < 1} {
		# copy will be the first visible element
		set neworderno 1
	} else {
		set maxorderno [getMaxOrderNo "$formid"]
		if {$neworderno > ($maxorderno + 1)} {
			# copy will be the last element in the questionary
			set neworderno [expr $maxorderno + 1]
		}
	}
	
	# copy the selected element, and insert the copy at its designated place
	set result [copyOrderNo "$formid" "$orderno" "$neworderno"]
	
	# mark the questionary as updated
	setAttribute "$formid" "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	
	if {$result} {
		# if copy-operation was successful, maybe some attributes have to 
		# be updated. values are provided in parameterlist
		
		# filter list with cgi-input for valid attributes
		set validlist [transformInput $parameterlist [getValidParametersForType "$elementtype"]]	
		
		# set all attributes provided - this is not counted for the result
		foreach {alias value} $validlist {
			setAttribute "$formid" "$alias" "$value" "$neworderno"
		}
		
		return [displayElementSuccessMsg [subst {
			Das Kopieren des Elementes war erfolgreich. Sie k&ouml;nnen das 
			duplizierte Element jetzt weiterbearbeiten.
		}]]
	} else {
		return [Error::db_Form_Copy_Operation_Failed]
	}
}


########################################################################


proc DBOperations::el_Delete	{	{formid {}}
											{password {}}
											{orderno {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# check the orderno of the element to be deleted, which must exist
	set checkresult [Util::check_OrderNo "$formid" "$orderno" "hastoexist"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	}
	
	# delete element from questionary, closing the gap this created
	set result [consistentDeleteOrderNo "$formid" "$orderno"]

	# mark the questionary as updated
	setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Das L&ouml;schen des Elementes war erfolgreich. Alle diesem 
			Element zugeordneten Daten wurden aus der Datenbank entfernt.
		}]]
	} else {
		return [Error::db_Delete_Operation_Failed]
	}
}


########################################################################


proc DBOperations::el_Edit	{	{formid {}}
										{password {}}
										{orderno {}}
										{type ""}
										{parameterlist {}}
									}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# check the orderno of the element to be edited, which must exist
	set checkresult [Util::check_OrderNo "$formid" "$orderno" "hastoexist"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# filter list with cgi-input for valid attributes
	set validlist [transformInput $parameterlist [getValidParametersForType "$type"]]
	
	# check all values supplied for attributes, and reject erroneous input
	set checkresult [checkInput $validlist]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# to keep questionary consistent, delete element first, and then
	# write it again from scratch
	set result [deleteOrderNo "$formid" "$orderno"]
	if {$result} {
		set result [insertOrderNo "$formid" "$orderno" $validlist]
	}

	# mark the questionary as updated
	setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Die neuen Daten fr dieses Element wurden erfolgreich 
			&uuml;bernommen.
		}]]
	} else {
		return [Error::db_Insert_Operation_Failed]
	}
}


########################################################################


proc DBOperations::el_Insert	{	{formid {}}
											{password {}}
											{orderno {}}
											{type ""}
											{parameterlist {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# check the orderno of the element to be copied
	set checkresult [Util::check_OrderNo "$formid" "$orderno" "nice"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# filter list with cgi-input for valid attributes
	set validlist [transformInput $parameterlist [getValidParametersForType "$type"]]

	# check all values supplied for attributes, and reject erroneous input
	set checkresult [checkInput $validlist]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# recheck orderno - if no elements visible, this will be the 1st element
	if {! [string is integer -strict "$orderno"]} {
		set orderno 1
	}
	
	# determine the position the element will be inserted at
	if {$orderno < 1} {
		set orderno 1
	} else {
		set maxorderno [getMaxOrderNo "$formid"]
		if {	($maxorderno > 0)
			&& ($orderno > $maxorderno)
		}	{
			set orderno $maxorderno
		}
	}

	# insert the new element into questinary, and move existing elements
	# away
	set result [consistentInsertOrderNo "$formid" "$orderno" $validlist]

	# mark the questionary as updated
	setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Das neue Element wurde erfolgreich an Position $orderno in 
			den Fragebogen aufgenommen.
		}]]
	} else {
		return [Error::db_Insert_Operation_Failed]
	}
}


########################################################################


proc DBOperations::el_Append	{	{formid {}}
											{password {}}
											{type ""}
											{parameterlist {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# filter list with cgi-input for valid attributes
	set validlist [transformInput $parameterlist [getValidParametersForType "$type"]]
	
	# check all values supplied for attributes, and reject erroneous input
	set checkresult [checkInput $validlist]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# new element will be the last element in line
	set orderno [expr "[getMaxOrderNo $formid] + 1"]
	set result [insertOrderNo "$formid" "$orderno" $validlist]
	
	# mark the questionary as updated
	setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Das neue Element wurde erfolgreich an das Ende des Fragebogens
			angeh&auml;ngt.
		}]]
	} else {
		return [Error::db_Insert_Operation_Failed]
	}
}


########################################################################


proc DBOperations::el_Move {	{formid {}}
										{password {}}
										{orderno {}}
										{neworderno {}}
									}	{
	
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# check the orderno of the element to be copied, which must exist
	set checkresult [Util::check_OrderNo "$formid" "$orderno" "hastoexist"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# determine the position where the element will be re-inserted
	if {$neworderno < 1} {
		set neworderno 1
	} else {
		set maxorderno [getMaxOrderNo "$formid"]
		if {$neworderno > ($maxorderno + 1)} {
			set neworderno [expr $maxorderno + 1]
		}
	}
	
	# if both positions are equal, there's nothing to be done
	if {$neworderno == $orderno} {
		set result 1
	} else {
		# simulate move as a sequence of copy and delete-operations
		set result [copyOrderNo "$formid" "$orderno" "$neworderno"]
		if {$result} {
			# recalculate the original position, in case the copy has
			# been inserted above the original
			if {$neworderno < $orderno} {
				set orderno [expr "$orderno + 1"]
			}
			# only delete if a copy was made - prevents data loss
			set result [consistentDeleteOrderNo "$formid" "$orderno"]
			if {! $result} {
				Generator::dbDebugMsg "DB-OPERATION FAILED: consistentDeleteOrderNo $formid $orderno -> $result"
			}
		} else {
			Generator::dbDebugMsg "DB-OPERATION FAILED: copyOrderNo $formid $orderno $neworderno -> $result"
		}
	
		# mark the questionary as updated
		setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	}
	
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Das Element wurde erfolgreich an die Position $neworderno
			verschoben.
		}]]
	} else {
		return [Error::db_Form_Copy_Operation_Failed]
	}
}


########################################################################


proc DBOperations::fbg_Copy	{	{sourceformid {}}
											{title {}}
											{password {}}
											{passwordnew {}}
											{passwordcheck {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$sourceformid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# check the new password that was provided against it's confirmation
	set checkresult [checkNewPassword "$passwordnew" "$passwordcheck"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	}
	
	# check the new title that was provided, which must not exist
	set checkresult [checkNewTitle "$title"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	}
	
	# generate a unique identifier for the copy
	set targetformid [Util::generate_FormID]
	
	# create a complete copy of the orignal, using the new formid
	set result [copyAllAttributes $sourceformid $targetformid]

	# use different password for the copy
	set result [expr "$result && [setPassword $targetformid $passwordnew]"]

	# use different title for the copy, to make it different
	setAttribute $targetformid "FRM_TITLE" "$title"
	
	# mark this glorious moment as the questionary's birthday
	setAttribute $targetformid "FRM_CREATED_ON" "[Util::get_Timestamp]"

	# mark the questionary as updated
	setAttribute $targetformid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displaySuccessMsg [subst {Das Kopieren des Fragebogens war 
			erfolgreich. Es wurden alle Datens&auml;tze in den neuen 
			Fragebogen mit dem Titel &quot;$title&quot; dupliziert.
		}]]
	} else {
		return [Error::db_Form_Copy_Operation_Failed]
	}
}


########################################################################


proc DBOperations::fbg_Delete { 	{formid {}}
											{password {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# delete all of the describing data completely
	set attributeresult [deleteAttribute $formid]
	
	# delete all of the entered results completely
	set sessionsresult [deleteFormSessions $formid]
	
	# success is, when both things were accomplished
	set result [expr "$attributeresult && $sessionsresult"]
	if {$result} {
		return [displaySuccessMsg {Das L&ouml;schen des Fragebogens aus der
			Datenbank war erfolgreich. Alle diesen Fragebogen betreffenden
			Datens&auml;tze wurden entfernt.
		}]
	} else {
		return [Error::db_Delete_Operation_Failed]
	}

}


########################################################################


proc DBOperations::fbg_Edit	{	{formid {}}
											{password {}}
											{title {}}
											{passwordnew {}}
											{passwordcheck {}}
											{parameterlist {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# only check and change password, if there was entered a value
	if {[string length "$passwordnew"] > 0} {
		# check the new password that was provided against it's confirmation
		set checkresult [checkNewPassword "$passwordnew" "$passwordcheck"]
		if {[string length "$checkresult"] > 0} {
			return $checkresult
		} 
	} else {
		set passwordnew $password
	}

	# remember old title even after deletion
	set oldtitle [get_Form_Title "$formid"]
		
	# if the old title's equal to the new one, there's nothing to be done 
	if	{	([string equal "$oldtitle" "$title"])
		||	([string length "$title"] < 1)
	}	{
		set title "$oldtitle"
	} else {
		# check the new title that was provided, which must not exist
		set checkresult [checkNewTitle "$title"]
		if {[string length "$checkresult"] > 0} {
			return $checkresult
		} 
	}

	# filter list with cgi-input for valid attributes
	set validlist [transformInput $parameterlist [getValidParametersForType "MISC"]]
	
	# check all values supplied for attributes, and reject erroneous input
	set checkresult [checkInput $validlist]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# editing the general info of a questionary works like editing an 
	# element: first delete this item completely, then write it again 
	# from scratch
	set result [deleteOrderNo "$formid" "0"]
	if {$result} {
		set result [insertOrderNo "$formid" "0" $validlist]
		set result [expr "$result && [setPassword $formid $passwordnew]"]
		set result [expr "$result && [setAttribute "$formid" "FRM_TITLE" "$title"]"]
	}

	# mark the questionary as updated
	setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
	if {$result} {
		return [displayElementSuccessMsg [subst {
			Die neuen Daten fr dieses Element wurden erfolgreich 
			&uuml;bernommen.
		}]]
	} else {
		return [Error::db_Insert_Operation_Failed]
	}
}


########################################################################


proc DBOperations::fbg_Generate { {formid {}} } {
	# a questionary with this formid must exist
	if {! [form_Exists "$formid"]} {
		return [Error::invalid_Form_Selected]
	} else {
		# read data from table and pass it on to the builder module		
		# initiate the building of the questionary
		set result [Builder::build "[getFormData $formid]" "[getMaxOrderNo $formid]"]
		return $result
	}
}


########################################################################


proc DBOperations::fbg_New {	{title {}}
										{password {}}
										{passwordcheck {}}
										{parameterlist {}}
									}	{
	# check the new password that was provided against it's confirmation
	set checkresult [checkNewPassword "$password" "$passwordcheck"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 

	# filter list with cgi-input for valid attributes
	set validparameters [transformInput $parameterlist [getValidParametersForType "MISC"]]

	# check the new title that was provided, which must not exist
	set checkresult [checkNewTitle "$title"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} else {
		# make the questionary's title a valid parameter
		lappend validparameters "FRM_TITLE" "$title"
	}
		
	# check all values supplied for attributes, and reject erroneous input
	set checkresult [checkInput $validparameters]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	# generate a unique identifier for this questionary
	set formid [Util::generate_FormID]

	# first of all, try to protect the questionary
	set insertresult [setPassword "$formid" "$password"]
	if {! $insertresult} {
		dbDebugMsg "DB-OPERATION FAILED: Insert of password for $formid failed!"
	}

	# insert every value which is recognized as valid
	set lastresult [insertOrderNo "$formid" "0" $validparameters]
	set insertresult [expr "$insertresult && $lastresult"]

	if {$insertresult} {
		# mark this glorious moment as the questionary's birthday
		setAttribute "$formid" "FRM_CREATED_ON" "[Util::get_Timestamp]"
		
		set msg [subst {Das Anlegen des neuen Fragebogens mit dem Titel
			&quot;$title&quot; war erfolgreich. Sie k&ouml;nnen diesen Fragebogen jetzt
			weiterbearbeiten, indem Sie ihn auf der Startseite 
			ausw&auml;hlen.
		}]
		return [displaySuccessMsg $msg]
	} else {
		return [Error::db_Insert_Operation_Failed]
	}
}


########################################################################


proc DBOperations::fbg_Rename	{	{formid {}}
											{title {}}
											{password {}}
										}	{
	# check the formid and password that were supplied								
	set checkresult [Util::check_FormID_And_Password "$formid" "$password"]
	if {[string length "$checkresult"] > 0} {
		return $checkresult
	} 
	
	set oldtitle [get_Form_Title "$formid"]
		
	# if the old title's equal to the new one, there's nothing to be done 
	if {! [string equal "$oldtitle" "$title"]} {
		# check the new title that was provided, which must not exist
		set checkresult [checkNewTitle "$title"]
		if {[string length "$checkresult"] > 0} {
			return $checkresult
		} else {
			# rename the questionary
			set result [setAttribute $formid "FRM_TITLE" "$title"]
			# mark the questionary as updated
			setAttribute $formid "FRM_LAST_UPDATED" "[Util::get_Timestamp]"
		}
	} else {
		# if you want to do nothing, this will always be successful
		set result 1
	}

	if {$result} {
		return [displaySuccessMsg [subst {Der Fragebogen wurde erfolgreich 
			in &quot;$title&quot; umbenannt.
		}]]
	} else {
		return [Error::db_Form_Copy_Operation_Failed]
	}
}


########################################################################


proc DBOperations::fbg_Result	{	{formid {}}
											{answers {}}
										}	{
	# questionary must exist
	if {! [form_Exists "$formid"]} {
		return [Error::invalid_Form_Selected]
	} else {
		set insertresult 1
		# make a new session
		set sessionid "[Util::generate_SessionID]"
		# figure out the remote adress of the sender
		set remoteaddress "[Util::get_Remote_Address]"
		# figure out which questions can be answered, and which input should
		# be stored in the database
		set validquestionnumbers [selectAttribute "$formid" "QST_NUMBER"]

		# view all questions as single questions
		foreach number $validquestionnumbers {
			# a questionnumber in the results table looks like qst23
			set qstnumber "qst$number"
			set answerlist {}
			# keep on looking for such a number in answers, and transfer
			# all answers with that number into a list
			while {[Util::is_In_List "$answers" "$qstnumber" "index"]} {
				lappend answerlist [lindex [lindex $answers $index] 1]
				# to avoid getting an infinite loop...
				set answers [lreplace $answers $index $index]
			}

			# if we found some answers to that question
			if {	([llength $answerlist] > 0)
				&& ([llength [lindex $answerlist 0]] > 0)
			}	{
				# delete NULL values from list - these are special
				Util::delete_From_List answerlist "NULL"
				# insert all answers into results table
				set lastresult [insertAnswer "$formid" "$sessionid" "$qstnumber" "$answerlist"]
				set insertresult [expr "$insertresult && $lastresult"]
			} else {
				# insert NULL value, as no answers were found
				insertAnswer "$formid" "$sessionid" "$qstnumber"
			}		
		}

		# try and view all numbers as blockquestions
		foreach number $validquestionnumbers {
			# blockquestions look like qst23-0 in the results table
			set globpattern "qst$number-*"
			# search answers for that kind of number
			while {[Util::is_In_List "$answers" "$globpattern" "index" 0 "glob"]} {
				# if we found something, keep that number
				set qstnumber [lindex [lindex $answers $index] 0]
				set answerlist {}
				# build a list of answers for that number
				while {[Util::is_In_List "$answers" "$qstnumber" "index"]} {
					lappend answerlist [lindex [lindex $answers $index] 1]
					# keep on deleting the items found
					set answers [lreplace $answers $index $index]
				}

				# if answers were found for that question
				if {	([llength $answerlist] > 0)
					&& ([llength [lindex $answerlist 0]] > 0)
				}	{
					# delete NULL values from list - these are special
					Util::delete_From_List answerlist "NULL"
					# insert all answers into results table
					set lastresult [insertAnswer "$formid" "$sessionid" "$qstnumber" "$answerlist"]
					set insertresult [expr "$insertresult && $lastresult"]
				} else {
					# insert NULL value, as no answers were found
					insertAnswer "$formid" "$sessionid" "$qstnumber"
				}		
			}
		}	

		# if saving of answers was successful
		if {$insertresult} {
			# enter this session into the resultlog table
			insertLog "$sessionid" "$remoteaddress"
			return [displaySuccessMsg [subst {
				<headline-page text="Vielen Dank!">
				Herzlichen Dank, da&szlig; Sie Sich die Zeit genommen und 
				die M&uuml;he gemacht haben, unsere Fragen zum Thema 
				&quot;[get_Form_Title "$formid"]&quot; zu beantworten. Ihre 
				Einsendung wurde erfa&szlig;t und wird in die Auswertung 
				eingehen.
			}] ]
		} else {
			# simulate rollback - don't save a half-completed questionary
			deleteAllAnswersFromSession "$formid" "$sessionid"
			return [Error::submission_Not_Accepted]
		}
	}
}


########################################################################


proc DBOperations::build_Form_List {} {
	set result {}

	set select "formid, value"
	set from "formdata"
	set where [subst {
		WHERE Attribute LIKE "[getAttributeName FRM_TITLE]"
		  AND orderno = 0
		ORDER BY value
	}]

	# read formids (= unique identifiers) and titles from database
	set rowcount [dbcselect "$select" "$from" "$where"]
	if { $rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			set rowresult [lindex $row 1]
			# append something in the form of "formid=title"
			lappend result "[lindex $rowresult 0]=[lindex $rowresult 1]"
			# fetch the next row
			set row [dbcfetch]
		}
	}

	return $result
}


########################################################################


proc DBOperations::build_Element_List { {formid {}} } {
	set result {}

	set select "orderno, attribute, value"
	set from "formdata"
	# only select items by which we can distinquish the different types
	# of questionary elements
	set where [subst {WHERE formid LIKE "$formid"
		AND (Attribute LIKE "[getAttributeName QST_NUMBER]"
		OR Attribute LIKE "[getAttributeName QST_TEXT]"
		OR Attribute LIKE "[getAttributeName QST_SHOW_NUMBER]"
		OR Attribute LIKE "[getAttributeName INP_SEL_SUB_QST_ALIGNMENT]"
		OR Attribute LIKE "[getAttributeName INP_TXT_WIDTH]"
		OR Attribute LIKE "[getAttributeName INP_SEL_TYPE]"
		OR Attribute LIKE "[getAttributeName TXT_TEXT]"
		OR Attribute LIKE "[getAttributeName TXT_ALIGNMENT]"
		OR Attribute LIKE "[getAttributeName SEP_TYPE]"
		OR Attribute LIKE "[getAttributeName SEP_DATA]")
		ORDER BY orderno
	}]
	set elementlist {}
	# submit query to database
	set rowcount [dbcselect "$select" "$from" "$where"]
	if { $rowcount > 0} {
		set row [dbcfetch]
		# while the row fetched contains data
		while {! [emptyRow $row]} {
			# the data we need is in the first list-element
			set selectresult "[lindex $row 1]"
			# extract different parts from data: see select above
			set orderno [lindex $selectresult 0]
			# we want to work with aliases, so do a reverse-lookup
			set alias [getAliasForAttribute [lindex $selectresult 1]]
			# new list is structured like {orderno {alias value}}
			set values [list $alias [lindex $selectresult 2]]
			lappend elementlist [list "$orderno" $values]
			set row [dbcfetch]
		}
	}
		
	# we need the length of the elementlist to know when we got all
	# elements described. as some elements can be described by more than
	# one list-item, we need to delete all items for an element, but 
	# don't know how many items these are, and where they are found...
	set listlength [llength $elementlist]
	set index 0
	# stop when all items have been described
	while {$index < $listlength} {
		# select an item from the elementlist
		set currentitem [lindex $elementlist $index]
		# get position of the element this item is describing
		set currentorderno [lindex $currentitem 0]
		# extract description, which is stored in a sub-list
		set sublist [Util::get_Sub_List $elementlist $currentorderno]
		# determine type of the element by looking at the description
		set type [Util::determine_Element_Type $sublist]
		# the different types will be describe differently - handle this
		switch -exact -- $type {
			QST_SELECT -
			QST_TEXT -
			QST_BLOCK {
				# preset description with default-values
				set text "{kein Text eingegeben}"
				set number "{keine Nummer vergeben}"
				# search the question's text
				if {[Util::is_In_List $sublist "QST_TEXT" searchindex]} {
					set textentry [lindex [lindex $sublist $searchindex] 1]
					# only override default if entry makes some sense
					if {[string length "$textentry"] > 0} {
						# no need to display the full question
						set text [Util::abbreviate "$textentry" 50 "..." 1]
					}
				}
				# search the question's number
				if {[Util::is_In_List $sublist "QST_NUMBER" searchindex]} {
					set numberentry [lindex [lindex $sublist $searchindex] 1]
					# only override default if entry makes some sense
					if {[string length "$numberentry"] > 0} {
						set number $numberentry
					}
				}
				# convert the element's type into something human-readable
				set designator [Util::element_Type_To_String "$type"]
				# describe the element like "1=1. Auswahlfrage 1) Warum?"
				lappend result "$currentorderno=$currentorderno. $designator $number) $text"
			}
			TEXT {
				# preset description with default-value
				set value "{kein Text eingegeben}"
				# search for the text
				if {[Util::is_In_List $sublist "TXT_TEXT" searchindex]} {
					set valueentry [lindex [lindex $sublist $searchindex] 1]
					# only override default if entry makes some sense
					if {[string length "$valueentry"] > 0} {
						# no need to display all the text, could be long
						set value [Util::abbreviate "$valueentry" 50 "..." 1]
					}
				}
				# describe the element like "2=2. Text: Blablabla..."
				lappend result "$currentorderno=$currentorderno. Text: $value"
			}
			SEPERATOR {
				# preset description with default-values
				set data "{keine zus&auml;tzlichen Daten}"
				set spacertype "{nicht bestimmbar}"
				# search for some data
				if {[Util::is_In_List $sublist "SEP_DATA" searchindex]} {
					set dataentry [lindex [lindex $sublist $searchindex] 1]
					# only override default if entry makes some sense
					if {[string length "$dataentry"] > 0} {
						set data $dataentry
					}
				}
				# search for the kind of spacer applied
				if {[Util::is_In_List $sublist "SEP_TYPE" searchindex]} {
					set typeentry [lindex [lindex $sublist $searchindex] 1]
					# only override default if entry makes some sense
					if {[string length "$typeentry"] > 0} {
						set spacertype [Util::spacer_Type_To_String "$typeentry"]
					}
				}
				# describe the element like "3=3. Trenner: Typ Bild - ./me.jpg
				lappend result "$currentorderno=$currentorderno. Trenner: Typ $spacertype - $data"
			}
			default {
				# can't determine the element's type with data provided
				lappend result "$currentorderno=$currentorderno. {Typ des Elements nicht bestimmbar}"
			}
		}
			
		# ok, we described this element, so delete everything about this
		# element from the list, to evade an endless loop
		set elementslost [Util::delete_From_List elementlist $currentorderno]
		# if no elements where deleted (shouldn't happen that often), go
		# to next item by incrementing the index, else recalculate the
		# length of the list, as it has changed
		if {$elementslost < 1} {
			incr index
		} else {
			set listlength [llength $elementlist]
		}
	}
	
	# result is a list containing many descriptions of elements (as strings)		
	return $result
}


########################################################################


proc DBOperations::db_Is_Open {} {
	variable db_is_open
	# if database wasn't opened already
	if {! $db_is_open} {
		# try to open it
		set db_is_open [expr "[dbcopen]"]
		# if this failed, try to get the reason for this error
		if {! $db_is_open} {
			dbDebugMsg "DB-OPERATION FAILED: dbcopen" 
		}
	}
	return $db_is_open
}


########################################################################


proc DBOperations::close_DB {} {
	variable db_is_open
	# if database is open
	if { $db_is_open } {
		# try to close it
		set db_is_open [expr ! "[dbcclose]"]
		# if this failed, try to get the reason for this error
		if {$db_is_open} {
			dbDebugMsg "DB-OPERATION FAILED: dbcclose"
		}
	}
	return $db_is_open
}


########################################################################


proc DBOperations::get_Form_Title {	{formid {}} } {
	# check if there is a chance for the questionary to exist
	if {[string length $formid] < 1} {
		# no - we don't have no unnamed questionaries
		return ""
	} else {
		# yes, might exist, let's look into the database
		set result [lindex [selectAttribute "$formid" "FRM_TITLE" 0] 0]
		return "$result"
	}
}


########################################################################


proc DBOperations::password_Matches	{	{formid {}}
													{password {}}
												}	{
	# check if there is a chance for the questionary to exist
	if {[string length $formid] < 1} {
		# no - we don't have no unnamed questionaries
		return 0
	} else {
		# yes, might exist, let's look into the database
		set attrname [getAttributeName FRM_PASSWORD]
		set column {VALUE}
		set table [getTableName TBL_FORMDATA]
		set condition [subst {WHERE FORMID LIKE "$formid"
									AND ATTRIBUTE LIKE "$attrname"
		}]
		# query database for the password stored there
		set rowcount [dbcselect $column $table $condition]
		# if there was a password stored in the database
		if { $rowcount > 0} {
			set row [dbcfetch]
			# build a list with password entries - normaly, there should 
			# only be 1 entry, but we can't know
			while {! [emptyRow $row]} {
				lappend result [lindex [lindex $row 1] 0]
				set row [dbcfetch]
			}
			# if the password provided by the user is not in the list,
			# it is considered WRONG, and the user should not be accepted
			return [expr "! ([lsearch -exact $result $password] == -1)"]
		} elseif	{$rowcount == 0} {
			# no password entry found, this questionary has no password
			# this is the case with old questionaries
			return 1
		} else {
			# never-reached-branch :-) just to make sure we are not tricked
			return 0
		}
	}
}


########################################################################


proc DBOperations::no_Visible_Elements	{ {formid {}} } {
	# check if the questionary is empty. this is the case if it has no
	# elements with a position higher 0. position 0 and less stores
	# (invisible) general data.
	set maxorderno [getMaxOrderNo $formid]
	return [expr "$maxorderno < 1"]
}


########################################################################


proc DBOperations::form_Exists	{ {formid {}} } {
	# check if there is a chance for the questionary to exist
	if {[string length "$formid"] < 1} {
		# no - we don't have no unnamed questionaries
		return 0
	} else {
		# yes, might exist, let's look into the database
		set title [get_Form_Title $formid]
		# a questionary must have a title to exist
		return [expr "[string length $title] > 0"]
	}
}


########################################################################


proc DBOperations::orderno_Exists	{	{formid {}}
													{orderno {}}
												}	{
	# select anything with this position from the questionary
	set resultlist [selectAttribute "$formid" "%" "$orderno"]
	# if items where found, the orderno exists
	return [expr "[llength $resultlist] > 0"]
}


########################################################################


proc DBOperations::get_Element_Type	{	{formid {}}
													{orderno {}}
												}	{
	# read the element's data from the database
	set resultlist [select_OrderNo "$formid" "$orderno"]
	# restructure the list from {alias1 value1 alias2 value2}
	# to {{alias1 value1} {alias2 value2}}
	foreach {alias value} $resultlist {
		lappend aliaslist [list "$alias" "$value"]
	}
	# determine the type of the element by it's describing data
	return [Util::determine_Element_Type $aliaslist]
}


########################################################################


proc DBOperations::select_OrderNo	{	{formid "%"}
													{orderno ""}
												}	{
	set result {}
	set column {ATTRIBUTE, VALUE}
	set table [getTableName TBL_FORMDATA]
	set condition [subst {WHERE FORMID LIKE "$formid"}]
	# if no correct orderno is provided, this will select all the 
	# questionaries data
	if {[string is integer -strict "$orderno"]} {
		append condition "AND ORDERNO = $orderno"
	}
	# select all data with this orderno from database
	set rowcount [dbcselect $column $table $condition]
	if {$rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			set rowcontent [lindex $row 1]
			# lookup the alias for this attribute
			set alias [getAliasForAttribute "[lindex $rowcontent 0]"]
			# quote values
			set value [special2html "[lindex $rowcontent 1]"]
			# build a list like {alias1 value1 alias2 value2}
			lappend result "$alias" "$value"
			set row [dbcfetch]
		}
	}

	return $result
}


########################################################################
##############################   Privat   ##############################
########################################################################


# There once was a young girl called Mary
# Who outwardly looked a bit scary
#     But she went on to mess
#     With theoretical math
# And unwillingly turned into a fairy


########################################################################


proc DBOperations::dbDebugMsg { {additionaltext {}} } {
	global dbcErrorMessage
	# advanced error-message including any error-messages from the 
	# database-adapter
	set dbctext [uplevel 1 Generator::debug_Msg [subst {
		"$additionaltext\nDB_Debug: $dbcErrorMessage"
	}]]
}


########################################################################


proc DBOperations::emptyRow { {row {}} } {
	# dissect row provided by dbcfetch to check if it contains data
	set elementcount [lindex $row 0]
	set rowcontent [lindex $row 1]
	# if elementcount is negative, this should be an error - log it
	if {$elementcount == -1} {
		dbDebugMsg "DB-OPERATION FAILED: Database-Error detected?!"
	}
	# row is empty if:	- it contains an empty list
	#							- the elementcount is below 1
	#							- it has no content
	# it is not empty if it has the empty content {} !
	return [expr	"	([llength $row] == 0) \
						||	($elementcount == 0) \
						||	($elementcount == -1) \
						||	([llength $rowcontent] == 0) \
						"]
}


########################################################################


proc DBOperations::getAttributeName { {name {}} } {
	# use formdata-attributes-array build by tableinfo.tcl to look up the 
	# corresponding entry to an alias
	variable formdata_attributes
	# alias may not be empty, and array must exist
	if {	([string length "$name"] > 0)
		&& ([array exists formdata_attributes])
	} {
		# all aliases are in capitals
		set uppercaseconstantname [string toupper "$name"]
		# try to lookup the entry by using the alias as an index
		if {[catch {set attribute $formdata_attributes($uppercaseconstantname)}]} {
			# if alias was not found, log the error, and use the alias as it's
			# own corresponding entry. this might work out ok.
			Generator::debug_Msg "$name: Map-Entry doesn't exist!"
			return "$name"
		}
		return "$attribute"
	} else {
		# either the array does not exist, or the name was empty
		Generator::debug_Msg "$name: Translation-Map doesn't exist!"
		# try using the alias - this might work out.
		return "$name"
	}
}


########################################################################


proc DBOperations::getAliasForAttribute { {attribute ""} } {
	# on first call, build a reverse-lookup-array to formdata_attributes
	# this is used later when needing an alias for a given attribute
	variable reverse_lookup
	variable reverse_lookup_exists
	
	# if this is the first time the procedure is called
	if {! $reverse_lookup_exists} {
		# build reverse-lookup-array from original array
		variable formdata_attributes
		foreach name [array names formdata_attributes] {
			set reverse_lookup($formdata_attributes($name)) $name
		}
		# remember that the reverse-lookup was already built
		set reverse_lookup_exists 1
	}
	
	# attribute may not be empty, and reverse-lookup-array must exist
	if {	([string length "$attribute"] > 0)
		&& ([array exists reverse_lookup])
	} {
		# try to look up the alias
		if {[catch {set alias $reverse_lookup($attribute)}]} {
			# if the attribute doesn't exist, use it as it's own alias, this
			# might work out ok
			Generator::debug_Msg "$attribute: Alias doesn't exist!"
			return "$attribute"
		}
		return "$alias"
	} else {
		# reverse-lookup-array doesn't exist, or no attribute specified
		Generator::debug_Msg "$attribute: Reverse-Lookup-Map doesn't exist!"
		# use the attribute as it's own alias, that might work.
		return "$attribute"
	}
}


########################################################################


proc DBOperations::getTableName { {name {}} } {
	global global_options
	# tablename may not be empty, and global-options-array must exist
	if {	([string length "$name"] > 0)
		&& ([array exists global_options])
	} {
		# try to look up the table's real name in global options array
		if {[catch {set tablename $global_options($name)}]} {
			Generator::debug_Msg "$name: Table doesn't exist!"
			return "$name"
		}
		return "$tablename"
	} else {
		# global-options-array doesn't exist, or no tablename specified
		return ""
	}
}


########################################################################


proc DBOperations::buildRefreshList { {omitParamtersList {}} }	{
	# refresh all input-parameters ...
	foreach pair "[CGI::get_Parameter_List]" {
		set name [lindex $pair 0]
		set value [lindex $pair 1]
		# ... if the have a non-empty value, and are not marked for
		# being omitted
		if	{	([string length "$value"] > 0)
			&&	(! [Util::is_In_List "$omitParamtersList" "$name"])
		}	{
			# we only need to memorize the name, and will look up the
			# value again later
			lappend resultlist "$name"
		}
	}
	return $resultlist
}


########################################################################


proc DBOperations::titleAlreadyInUse { {title {}} } {
	# reject empty titles
	if {[string length "$title"] == 0} {
		return 1
	} else {
		set select "FORMID"
		set from [getTableName TBL_FORMDATA]
		set where "WHERE Attribute LIKE '[getAttributeName FRM_TITLE]'
			AND Value LIKE '$title'"
		# check if the title is used anywhere
		return [dbcselect "$select" "$from" "$where"]
	}
}	


########################################################################


proc DBOperations::insertLog	{	{sessionid {}}
											{remoteaddress {}}
										}	{
	# sessionid and remoteaddress must be set
	if	{	([string length $sessionid] < 1)
		||	([string length $remoteaddress] < 1)
	}	{
		return 0
	} else {
		set table [getTableName RESULTLOG]
		set columns "SESSIONID, REMOTEHOST"
		set values [subst {"$sessionid", "$remoteaddress"}]
		# insert a row into the resultlog table
		set result [dbcinsert $columns $table $values] 
		return [expr "$result == 0"]
	}
}	


########################################################################


proc DBOperations::insertAnswer	{	{formid {}}
												{sessionid {}}
												{qstnumber 0}
												{answer {}}
											}	{
	# formid and sessionid must be set
	if	{	([string length $formid] < 1)
		||	([string length $sessionid] < 1)
	}	{
		return 0
	} else {
		set table [getTableName TBL_RESULTS]
		set columns "FORMID, SESSIONID, QUESTIONNO, ANSWER"
		# an empty answer will be saved as NULL in the database
		if {[string length "$answer"] < 1} {
			set values [subst {"$formid", "$sessionid", "$qstnumber", NULL}]
		} else {
			set values [subst {"$formid", "$sessionid", "$qstnumber", "$answer"}]
		}
		# insert a row into the results table, containing an answer
		set result [dbcinsert $columns $table $values]

		return [expr "$result == 0"]
	}
}	


########################################################################


proc DBOperations::insertOrderNo	{	{formid {}}
												{orderno {}}
												{attributelist {}}
											}	{
	set result 1
	# attributelist is formatted like {alias1 value1 alias2 value2}
	foreach {alias value} $attributelist {
		# insert item at the orderno provided
		set lastresult [insertAttribute $formid $alias $value $orderno]
		# if this failed, try to log this error
		if {! $lastresult} {
			dbDebugMsg "DB-OPERATION FAILED: insertAttribute $formid $alias $value $orderno"
		}
		set result [expr "$result && $lastresult"]
	}

	return $result
}


########################################################################


proc DBOperations::consistentInsertOrderNo	{	{formid {}}
																{orderno {}}
																{attributelist {}}
															}	{
	# move away other elements to create room for the element to be 
	# inserted, as described by the attributelist
	set result [moveOrderNo "$formid" "$orderno" ">="]
	if {$result} {
		# insert the new element
		set insertresult [insertOrderNo $formid $orderno $attributelist]
		# if this failed, try to log this error
		if {! $insertresult} {
			Generator::dbDebugMsg "DB-OPERATION FAILED: insertOrderNo $formid $orderno $attributelist"
		}
		set result [expr "$result && $insertresult"]
	} else {
		Generator::dbDebugMsg "DB-OPERATION FAILED: moveOrderNo $formid $orderno >="
	}
	return $result
}


########################################################################


proc DBOperations::insertAttribute	{	{formid {}}
													{name {}}
													{value {}}
													{orderno 0}
												}	{
	set attrname [getAttributeName $name]
	# formid and attrname (name of the attribute) must be set
	if	{	([string length $formid] < 1)
		||	([string length $attrname] < 1)
	}	{
		return 0
	} else {
		set table [getTableName TBL_FORMDATA]
		set columns "FORMID, ORDERNO, ATTRIBUTE, VALUE"
		set values [subst {"$formid", "$orderno", "$attrname", "$value"}]
		# insert this item into the database
		set result [dbcinsert $columns $table $values] 
		return [expr "$result == 0"]
	}
}	


########################################################################


proc DBOperations::updateAttribute	{	{formid "%"}
													{name "%"}
													{value {}}
													{orderno ""}
												}	{
	set table [getTableName TBL_FORMDATA]
	# if no wildcard was used, lookup the attribute name
	if {! [string equal "%" "$name"]} {
		set attrname [getAttributeName $name]
		if {[string length $attrname] < 1} {
			return 0
		}
	} else {
		# if wildcard was used, select by wildcard
		set attrname "$name"
	}
	set namevalue [subst {VALUE="$value"}]
	set condition [subst {WHERE FORMID LIKE "$formid"
									AND ATTRIBUTE LIKE "$attrname"
	}]
	# if orderno is valid, use it in the update-statement
	if {[string is integer -strict "$orderno"]} {
		append condition "AND ORDERNO = $orderno"
	}
	set result [dbcupdate $table $namevalue $condition]
	return [expr "$result == 0"]
}	


########################################################################


proc DBOperations::deleteFormSessions { {formid ""} } {
	# a bit complicated: we need to delete the sessions from the 
	# resultlog table, but have only the formid. so we have to find out
	# which sessionid belong to this questionary from the results table
	set table [getTableName TBL_RESULTS]
	set column "distinct SESSIONID"
	set condition [subst {WHERE FORMID LIKE "$formid"}]

	set sessionids {}
	# select all sessionids which correspond to the given formid
	set rowcount [dbcselect $column $table $condition]
	if { $rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			lappend sessionids [lindex [lindex $row 1] 0]
			set row [dbcfetch]
		}
	}
	# delete all these entries from the results table
	set result [expr "[dbcdelete $table $condition] == 0"]
	set table [getTableName RESULTLOG]
	# delete the collected sessionids from the resultlog table
	foreach sessionid $sessionids {
		set condition [subst {WHERE sessionid LIKE "$sessionid"}]
		set lastresult [expr "[dbcdelete $table $condition] == 0"]
		set result [expr "$result && $lastresult"]
	}
	
	return $result
}


########################################################################


proc DBOperations::deleteAttribute	{	{formid "%"}
													{name "%"}
													{orderno ""}
												}	{
	set table [getTableName TBL_FORMDATA]
	if {! [string equal "%" "$name"]} {
		# if no wildcard is used, lookup the attributes real name
		set attrname [getAttributeName $name]
		# ban unnamed attributes
		if {[string length $attrname] < 1} {
			return 0
		}
	} else {
		# wildcard was used - keep it
		set attrname "$name"
	}
	set condition [subst {WHERE FORMID LIKE "$formid"
									AND ATTRIBUTE LIKE "$attrname"
	}]
	# if a valid orderno was provided, use it in the where-condition
	if {[string is integer -strict "$orderno"]} {
		append condition "AND ORDERNO = $orderno"
	}
	# delete everything matching the condition
	set result [dbcdelete $table $condition]
	return [expr "$result == 0"]
}	


########################################################################


proc DBOperations::deleteOrderNo	{	{formid "%"}
												{orderno ""}
											}	{
	set table [getTableName TBL_FORMDATA]
	set condition [subst {WHERE FORMID LIKE "$formid"}]
	# if a valid orderno was provided, use it in the where-condition
	if {[string is integer -strict "$orderno"]} {
		append condition "AND ORDERNO = $orderno"
	}
	# delete everything matching the where-condition
	set result [dbcdelete $table $condition]
	return [expr "$result == 0"]
}	


########################################################################


proc DBOperations::consistentDeleteOrderNo	{	{formid "%"}
																{orderno ""}
															}	{
	# delete everything with a matching orderno from questionary
	set result [deleteOrderNo "$formid" "$orderno"]
	if {$result} {
		# if delete was successful, close the gap we just created
		set result [moveOrderNo "$formid" "$orderno" "<="]
	}
	return $result
}


########################################################################


proc DBOperations::selectAttribute	{	{formid "%"}
													{name "%"}
													{orderno ""}
												}	{
	set result {}
	set column {VALUE}
	set table [getTableName TBL_FORMDATA]

	if {! [string equal "%" "$name"]} {
		# if no wildcard was used for the alias, get real name for it
		set attrname [getAttributeName $name]
		if {[string length $attrname] < 1} {
			# we don't do unnamed attributes
			return {}
		}
	} else {
		# wildcard was used - keep it as a real name
		set attrname "$name"
	}
	set condition [subst {WHERE FORMID LIKE "$formid"
									AND ATTRIBUTE LIKE "$attrname"
	}]
	# if a valid orderno was provided, use it in the where-statement
	if {[string is integer -strict "$orderno"]} {
		append condition "AND ORDERNO = $orderno"
	}
	# select all the attributes' values matching this condition
	set rowcount [dbcselect $column $table $condition]
	if { $rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			lappend result [lindex [lindex $row 1] 0]
			set row [dbcfetch]
		}
	}

	return $result
}


########################################################################


proc DBOperations::copyOrderNo	{	{formid {}}
												{sourceorderno {}}
												{targetorderno {}}
											}	{
	# sourcordnerno, targetorderno and formid must be set
	if	{	([string is integer -strict $sourceorderno])
		&&	([string is integer -strict $targetorderno])
		&&	([string length $formid] > 0)
	}	{
		# get a list of items to be copied for this element
		set tobecopiedlist [select_OrderNo $formid $sourceorderno]
		# insert this items at the position specified by targetorderno
		set result [consistentInsertOrderNo "$formid" "$targetorderno" $tobecopiedlist]
		return $result
	} else {
		return 0
	}

}


########################################################################


proc DBOperations::copyAllAttributes	{	{sourceformid {}}
														{targetformid {}}
													}	{
	set result 1

	set attributenamelist {}
	set column {ORDERNO, ATTRIBUTE, VALUE}
	set table [getTableName TBL_FORMDATA]
	set condition [subst {WHERE FORMID LIKE "$sourceformid"}]
	# we can get all the items of a questionary in one go ...
	set rowcount [dbcselect $column $table $condition]
	if { $rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			lappend attributenamelist [lindex $row 1]
			set row [dbcfetch]
		}
	}
	# ... but we need to insert them one by one
	foreach triplet $attributenamelist {
		# disect the list being the item
		set orderno [lindex $triplet 0]
		set name [lindex $triplet 1]
		set value [lindex $triplet 2]
		# insert the item in the target questionary
		set lastresult [insertAttribute $targetformid $name $value $orderno]
		set result [expr "$result && $lastresult"]
	}
	return $result
}


########################################################################


proc DBOperations::deleteAllAnswersFromSession	{	{formid {}}
																	{sessionid {}}
																}	{
	# formid and sessionid must be set
	if	{	([string length $formid] < 1)
		||	([string length $sessionid] < 1)
	}	{
		return 0
	} else {
		set table [getTableName TBL_RESULTS]
		set condition [subst {WHERE FORMID LIKE "$formid"
									AND SESSIONID LIKE "$sessionid"
		}]
		# delete all answers from results table matching this description
		set result [dbcdelete $table $condition]
		return [expr "$result == 0"]
	}
}


########################################################################


proc DBOperations::displaySuccessMsg { {msg {}} } {
	if {[string length "$msg"] == 0} {
		set msg "Die Datenbankoperation war erfolgreich."
	}
	# display message showing our success
	return [Generator::build_Page "Datenbankoperation erfolgreich" "<body-defaultnavigation>" "$msg"]
}		


########################################################################


proc DBOperations::displayElementSuccessMsg { {msg {}} } {
	if {[string length "$msg"] == 0} {
		set msg "Die Datenbankoperation war erfolgreich."
	}
	# display message showing our success
	return [Generator::build_Page "Datenbankoperation erfolgreich" "<body-elementnavigation>" "$msg"]
}		


########################################################################


proc DBOperations::checkNewTitle	{	{title {}}
											}	{
	# title must not be empty
	if {[string length "$title"] == 0} {
		set refreshlist [buildRefreshList "FRM_TITLE"]
		return [Error::no_Title_Provided $refreshlist]
	}
	# title must not be already in use
	if { [titleAlreadyInUse "$title"] } {
		set i 2
		# make a new suggestion for the title like "oldtitle (2)"
		set newtitle "$title ($i)"
		while { [titleAlreadyInUse "$newtitle"] } {
			incr i
			set newtitle "$title ($i)"
		}
		set refreshlist [buildRefreshList "FRM_TITLE"]
		return [Error::title_Already_In_Use $refreshlist $title $newtitle]
	}
	# everything ok about the title
	return ""
}															


########################################################################


proc DBOperations::checkNewPassword	{	{password {}}
													{passwordcheck {}}
												}	{
	# password must be longer than 5 chars
	if {[string length "$password"] < 6} {
		set refreshlist [buildRefreshList {FRM_PASSWORD_NEW FRM_PASSWORD_CHECK}]
		return [Error::password_To_Short $refreshlist]
	}
	# password and confirmation must not differ
	if {! [string equal "$password" "$passwordcheck"]} {
		set refreshlist [buildRefreshList {FRM_PASSWORD_NEW FRM_PASSWORD_CHECK}]
		return [Error::passwords_Not_Matching $refreshlist]
	}
	# everything ok about the password
	return ""
}															


########################################################################


proc DBOperations::setPassword	{	{formid {}}
												{password {}}
											}	{
	# formid must be set, and password must be longer 5 chars
	if	{	([string length "$formid"] > 0)
		&&	([string length "$password"] > 5)
	}	{
		# set the password attribute
		set result [setAttribute "$formid" "FRM_PASSWORD" "$password"]
		return [expr "$result"]
	} else {
		return 0
	}
}


########################################################################


proc DBOperations::setAttribute	{	{formid {}}
												{name {}}
												{value {}}
												{orderno 0}
											}	{
	# formid and the name of the attribute must be set
	if	{	([string length $formid] < 1)
		||	([string length $name] < 1)
	}	{
		return 0
	} else {
		if {[attributeExists $formid $name $orderno]} {
			# if attribute exists already, just update it
			set result [updateAttribute $formid $name $value $orderno]
		} else {
			# if attribute doesn't exist, insert it
			set result [insertAttribute $formid $name $value $orderno]
		}
		return [expr "$result"]
	}
}	


########################################################################


proc DBOperations::attributeExists	{	{formid {}}
													{name {}}
													{orderno ""}
												}	{
	# check for existance of attribute by finding it - or not finding it
	set appearances [selectAttribute $formid $name $orderno]
	return [expr "[llength $appearances] > 0"]
}


########################################################################


proc DBOperations::getMaxOrderNo { {formid} } {
	set result 0
	set column {max(ORDERNO)}
	set table [getTableName TBL_FORMDATA]
	set condition [subst {WHERE FORMID LIKE "$formid"}]
	# get maximum orderno for this questionary
	set rowcount [dbcselect $column $table $condition]
	if {$rowcount > 0} {
		set row [dbcfetch]
		while {! [emptyRow $row]} {
			set rowcontent [lindex $row 1]
			set tempresult [lindex $rowcontent 0]
			# return first result that is an integer
			if {[string is integer -strict "$tempresult"]} {
				set result $tempresult
			}
			set row [dbcfetch]
		}
	}
	return $result
}


########################################################################


proc DBOperations::moveOrderNo	{	{formid ""}
												{orderno 0}
												{comparator ">"}
											}	{
	# comparator signifies the direction of the movement
	if	{[expr "0 $comparator 1"]} {
		# if comparator is "less or equal" move all elements down one step,
		# for instance to close a gap
		set modificator "-1"
		# start with first element specified, to avoid owerwriting things
		set start $orderno
		# end at last element
		set end  [getMaxOrderNo "$formid"]
		# count from low to high orderno
		set increment 1
	} else {
		# if comparator is "bigger or equal", move all elements up one step, 
		# for instance to create room for a new element
		set modificator "+1"
		# start with last element, to avoid overwriting things
		set start [getMaxOrderNo "$formid"]
		# end at element specified
		set end $orderno
		# count from high to low orderno
		set increment -1
	}
	set table [getTableName TBL_FORMDATA]
	set result 1
	# iterate through elements as specified above
	for {set currentorderno $start} {[expr "$currentorderno $comparator $end"]} {incr currentorderno $increment} {
		# calculate the new orderno, that is, position
		set neworderno [expr "$currentorderno $modificator"]
		set namevalue "ORDERNO = $neworderno"
		set condition [subst {WHERE FORMID LIKE "$formid"
									AND ORDERNO = $currentorderno
		} ]
		# move the current element to it's new position (one up or down)
		set lastresult [dbcupdate $table $namevalue $condition]
		set result [expr "$result && ($lastresult == 0)"]
	}
	return $result
}	


########################################################################


proc DBOperations::getFormData { {formid {}} } {
	if {[string length $formid] < 1} {
		return {}
	} else {
		set result {}
		set column {ORDERNO, ATTRIBUTE, VALUE}
		set table [getTableName TBL_FORMDATA]
		set condition [subst {WHERE FORMID LIKE "$formid"}]
		set rowcount [dbcselect $column $table $condition]
		if { $rowcount > 0} {
			set row [dbcfetch]
			while {! [emptyRow $row]} {
				set entry [lindex $row 1]
				set alias [getAliasForAttribute [lindex $entry 1]]
				set quotedvalue [special2html [lindex $entry 2]]
				set value [list "$alias" "$quotedvalue"]
				lappend result "[lindex $entry 0]" "$value"
				set row [dbcfetch]
			}
		}

		return $result
	}
}
		

########################################################################


proc DBOperations::transformInput	{	{cgiparameterlist {}}
													{validparameterlist {}}
												}	{
	set result {}
	# iterate through cgiparameterlist, formatted like 
	# {{name1 value1} {name2 value2}}
	foreach namevaluepair $cgiparameterlist {
		set name [lindex $namevaluepair 0]
		set value [lindex $namevaluepair 1]
		# build a new list from all parameters marked as valid,
		# formatted like {name1 value1 name2 value2}
		if {[Util::is_In_List $validparameterlist "$name"]} {
			lappend result $name $value
		}
	}	
	return $result
}


########################################################################


proc DBOperations::getValidParametersForType { {elementtype ""} } {
	# valid parameters for all questions, can be inserted safely
	set questionscommonparameters [list "QST_NUMBER" "QST_SHOW_NUMBER" "QST_TEXT" "QST_ALIGNMENT" "QST_ANSWER_ALIGNMENT"]
	# valid parameters for all select-questions, can be inserted safely
	set selectquestionscommonparameters [list "INP_SEL_TYPE" "INP_SEL_DEFAULT" "INP_SEL_OPTION_VALUES"] 
	switch -exact -- $elementtype {
		QST_SELECT {
			# special parameters for select questions
			set parameters [list "INP_SEL_ALIGN_TEXT" "INP_SEL_ALIGN_OPTIONS"]
			return [concat $questionscommonparameters $selectquestionscommonparameters $parameters]
		}
		QST_TEXT {
			# special parameters for text questions
			set parameters [list "INP_TXT_LENGTH" "INP_TXT_WIDTH" "INP_TXT_HEIGHT"]
			return [concat $questionscommonparameters $parameters]
		}
		QST_BLOCK {
			# special parameters for block questions
			set parameters [list "INP_SEL_BLOCK_VALUES" "INP_SEL_SUB_QST_ALIGNMENT"]
			return [concat $questionscommonparameters $selectquestionscommonparameters $parameters]
		}
		QST_UNKNOWN { return {} }
		TEXT {
			# special parameters for text blocks
			return [list "TXT_TEXT" "TXT_ALIGNMENT" "TXT_SIZE" "TXT_COLOR" "TXT_LINK_TARGET"]
		}
		SEPERATOR {
			# special parameters for seperators
			return [list "SEP_TYPE" "SEP_DATA"]
		}
		MISC {
			# special parameters for the general description of a form
			return [list "FRM_DESCRIPTION" "FRM_CREATED_BY" "FRM_SHOW_TITLE" "FRM_BG_COLOR" "FRM_FG_COLOR"]
		}
		UNKNOWN { return {} }
		default { return {} }
	}
}


########################################################################


proc DBOperations::checkInput	{	{cgiparameterlist {}}
										}	{
	set result ""
	# as we need both type AND data to decide wether the data is valid,
	# we must cache both values here, and let them be set and unset 
	# by the checking procedure, DBOperations::checkParameter
	set seperatortype ""
	set seperatorcache ""
	# the new input-element will also be set by the checking procedure
	set newinputelement ""
	# check the whole list, and stop at the first error
	foreach {name value} $cgiparameterlist {
		set result [checkParameter "$name" "$value"]
		if {[string length $result] > 1}	{
			# i know this doesn't look like structured programming, but
			# imagine having to index this list - way to slow
			break
		}
	}
	# check if there was an error encountered
	if {[string length $result] > 1} {
		# if we were stopped by an error, refresh all cgi-input-
		# parameters but the erroneous one
		set refreshlist [buildRefreshList "$name"]
		# return this error-page
		set result [subst {
			<style style="page.style">
			<page-defaultbegin title="Fehlerhafte Eingabe">
			<body-navigation>
				<body-defaultnavigation>
			</body-navigation>
			<body-content>
				<body-paragraph>
					$result
				</body-paragraph>
				<form-begin>
					<form-parameterrefresh items="{$refreshlist}">
					<form-element label="Eingabe wiederholen:" valign="top">
						$newinputelement
					</form-element>
					<form-defaultbuttons>
				<form-end>
				<basic-divider>
				<static-indexlink>
			</body-content>
			<page-defaultend>
		}]
	}
	# if all parameters were ok, result should still be empty
	return $result
}


########################################################################


proc DBOperations::checkParameter	{	{name ""}
													{value ""}
												}	{
	set defaultmsg [subst {Der von Ihnen angegebene Wert "$value" ist 
		ungltig, denn er entstammt nicht der zugrundeliegenden Wertemenge.
	}]
	# get the reference on the new input-element to be created
	upvar 1 newinputelement input
	# initialize the input-element
	set input ""
	# initialize the result
	set message ""
	# we decide by the name of the parameter how it's value should look like
	switch -exact -- "$name" {
		QST_NUMBER -
		INP_TXT_LENGTH -
		INP_TXT_WIDTH -
		INP_TXT_HEIGHT {
			# these must be numbers, and may not be empty
			set message [Util::check_Number "$value"]
			set input [subst {<form-text cginame="$name" value="$value">}]
		}
		
		TXT_ALIGNMENT -
		QST_ALIGNMENT -
		INP_SEL_SUB_QST_ALIGNMENT {
			# these must either be left, center or right
			set valid [list left center right]
			if {! [Util::is_In_List $valid "$value"]} {
				set message $defaultmsg
				set input [subst {<static-alignmentdropdown cginame="$name" default="$value">}]
			}
		}
		
		TXT_COLOR -
		FRM_BG_COLOR -
		FRM_FG_COLOR {
			# these must be a named color or a hex-value formatted like "#rrggbb"
			set message [Util::check_Color "$value"]
			set input [subst {<static-color cginame="$name" value="$value">}]
		}
		
		TXT_SIZE {
			# this must be a number in the range of -5 to 5
			set message [Util::check_Number "$value" "-5" "5"]
			set input [subst {<form-dropdown cginame="TXT_SIZE" items="
					{-5=Winzig}
					{-4=Sehr, sehr klein}
					{-3=Sehr klein}
					{-2=Klein}
					{-1=Etwas kleiner}
					{0=Normal}
					{1=Etwas gr&ouml;&szlig;er}
					{2=Gro&szlig;}
					{3=Sehr gro&szlig;}
					{4=Sehr, sehr gro&szlig;}
					{5=Riesig}
				" default="$value">
			}]
		}
		
		SEP_TYPE {
			# this must be one of the following: hr, space or img
			set valid [list hr space img]
			set message [Util::is_In_List $valid "$value"]
			set input  [subst {<form-dropdown cginame="SEP_TYPE" items="
					{hr=Balken}
					{space=Zwischenraum}
					{img=Bild}
				" default="$value">
			}]
			# if the type was ok
			if {[string length $message] < 1} {
				# in a somewhat complicated way, check if data is matching for type
				upvar 1 seperatortype type
				upvar 1 seperatorcache cache
				if {[string equal "img" "$value"]} {
					# img needs an url
					if {[string length "$cache"] > 0} {
						set message [Util::check_URL "$cache"]
						set input [subst {<form-text cginame="SEP_DATA" value="$cache">}]
						set cache ""
						set type ""
					} else {
						set type "$value"
					}
				} else {
					# space and hr need a number
					if {[string is integer -strict "$cache"] > 0} {
						set message [Util::check_Number "$cache" 0]
						set input [subst {<form-text cginame="SEP_DATA" value="$cache">}]
						set type ""
					} else {
						set type "$value"
					}
					set cache ""
				}
			}
		}
		
		SEP_DATA {
			# in a less complicated way as above, check if type is matching data
			upvar 1 seperatortype type
			upvar 1 seperatorcache cache
			switch -exact -- "$type" {
				img {
					set message [Util::check_URL "$value"]
					set input [subst {<form-text cginame="SEP_DATA" value="$value">}]
					set cache ""
					set type ""
				}
				hr -
				space {
					set message [Util::check_Number "$value" 0]
					set input [subst {<form-text cginame="SEP_DATA" value="$value">}]
					set cache ""
					set type ""
				}
				default {set cache "$value"}
			}		
		}
		
		QST_ANSWER_ALIGNMENT {
			# this must either be aaar, aaal or baar
			set valid [list aaar aaal baar]
			if {! [Util::is_In_List $valid "$value"]} {
				set message $defaultmsg
				set input [subst {<form-dropdown cginame="QST_ANSWER_ALIGNMENT" items="
						{aaal=Linksb&uuml;ndig}
						{aaar=Rechtsb&uuml;ndig}
						{baar=Blockausrichtung}
					" default="$value">
				}]
			}
		}
		
		INP_SEL_TYPE {
			# this has to be radio, kombi, lists, listm or check
			set valid [list radio kombi lists check listm]
			if {! [Util::is_In_List $valid "$value"]} {
				set message $defaultmsg
				set input [subst {<form-dropdown cginame="INP_SEL_TYPE" items="
					{radio=Radiobuttons}
					{check=Checkboxes}
					{kombi=Dropdown-Liste}
					{lists=Listbox (Einfachauswahl)}
					{listm=Listbox (Mehrfachauswahl)}
				" default="$value">
			}]
			}
		}
		
		INP_SEL_ALIGN_TEXT {
			# this is either behind, before, above or below
			set valid [list behind before above below]
			if {! [Util::is_In_List $valid "$value"]} {
				set message $defaultmsg
				set input [subst {<form-dropdown cginame="INP_SEL_ALIGN_TEXT" items="
						{behind=Hinter der Schaltfl&auml;che}
						{before=Vor der Schaltfl&auml;che}
						{above=&Uuml;ber der Schaltfl&auml;che}
						{below=Unter der Schaltfl&auml;che}
					" default="$value">
				}]
			}
		}

		INP_SEL_ALIGN_OPTIONS {
			# this is either... - i think you figure it, else look above.
			set valid [list horizontal vertical]
			if {! [Util::is_In_List $valid "$value"]} {
				set message $defaultmsg
				set input [subst {<form-dropdown cginame="INP_SEL_ALIGN_OPTIONS" items="
					{horizontal=Horizontal}
					{vertical=Vertikal}
				" default="$value">
			}]
			}
		}

		FRM_SHOW_TITLE -
		QST_SHOW_NUMBER {
			# these must be booleans
			set message [Util::check_Boolean "$value"]
			set input [subst {<form-dropdown cginame="$name" items="
					{ja=Ja}
					{nein=Nein}
				" default="$value">
			}]
		}

		TXT_LINK_TARGET -
		INP_SEL_DEFAULT {
			# these must be integers, but may be empty
			if {! [string is integer "$value"]} {
				set message [Util::check_Number "$value"]
				set input [subst {<form-text cginame="$name" value="$value">}]
			}
		}
		default {
			# all other elements don't need to be checked, thus they are
			# always ok
			set input ""
			return "" 
		}
	}
	# guote where needed
	return [special2html $message]
}


########################################################################

