# -----------------------------------------------------------------------------
# FILE:    dbc-mysql.tcl
#
# PURPOSE: MySQL-based functionality for database connectivity 
#
# PROCS:   dbcopen     - open database
#          dbcclose    - close database
#          dbcselect   - select statement
#          dbcfetch    - fetch next row after select-statement
# ----------------------------------------------------------------------------


# ----------------------------------------------------------------------------
# GLOBAL DATA:
set dbHandle ""
set dbResultHandle ""

# ----------------------------------------------------------------------------
# PROC:    dbcopen
# PARAMS:  dbName, dbUser   via "global_options"
# RETURN:  0    - ok
#          else - error (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcopen {} {
    global dbHandle dbcErrorMessage global_options

    set dbName $global_options(DBNAME)
    set dbUser $global_options(DBUSER)
    set dbHost $global_options(DBHOST)
    set dbPass $global_options(DBPASS)

    trc dbcopen "begin (dbName='$dbName'; dbUser='$dbUser'; dbHost='$dbHost'; dbPass='$dbPass')"

    # get DB handle
    if { [catch {set dbHandle [sql connect $dbHost $dbUser $dbPass]} dbcErrorMessage] } {
	return 0
    }
    trc dbcopen "... connected to database-server"

    if { [catch {sql selectdb $dbHandle $dbName} dbcErrorMessage] } {
	return 0
    }

    trc dbcopen "database opened"
    return 1
}


# ----------------------------------------------------------------------------
# PROC:    dbcclose
# PARAMS:  --
# RETURN:  0   - ok
#          else - error (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcclose {} {
    global dbHandle dbcErrorMessage

    trc dbcclose "begin"
    
    if { [catch {sql disconnect $dbHandle} dbcErrorMessage] } {
        return 0
    }
    trc dbcclose "end"
    return 1
}


# ----------------------------------------------------------------------------
# PROC:    dbselect
# PARAMS:  cols      I   cols as SQL-string (e.g. "col_1, col_2")
#          tables    I   tables as SQL-string (e.g. "table_1, table_2")
#          condition I   string containing the condition (e.g. "WHERE ...")
# RETURN:  >0   - number of rows in result
#          <0   - error (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcselect {cols tables condition} {
    global dbResultHandle dbHandle dbcErrorMessage global_options

    set stmt "select $cols from $tables $condition"
    set global_options(DBCSTMT) $stmt

    if { [catch {set dbResultHandle [sql query $dbHandle "$stmt"]} dbcErrorMessage] } { 
	return -1
    }
    if { [catch {set numRows [sql numrows $dbHandle $dbResultHandle]} dbcErrorMessage] } {   
	return -1
    }
    return $numRows
}


# ----------------------------------------------------------------------------
# PROC:    dbcfetch
# PARAMS:  --
# RETURN:  1 row    row fetched
#          0        no more rows
#         -1        error (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcfetch {} {
    global dbResultHandle dbHandle dbcErrorMessage

    set resultList ""
    if { [catch {set r [sql fetchrow $dbHandle $dbResultHandle]} dbcErrorMessage] } {
        lappend resultList -1 {}
    } else {
        if { "$r" == "" } { 
            sql endquery $dbHandle $dbResultHandle 
	    lappend resultList 0 {}
        } else {
            lappend resultList 1 $r
        }
    }

    trc dbcfetch "$resultList"

    return $resultList
}

# ----------------------------------------------------------------------------
# PROC:    dbcinsert
# PARAMS:  cols      I   cols to insert values into
#          table     I   table to insert row
#          values    I   values for cols
# RETURN:  0        no error
#         <>0       error-code (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcinsert {cols table values} {
    global dbHandle dbcErrorMessage dbcErrorCode global_options

    trc dbcinsert "begin"

    set code $dbcErrorCode(NO_ERROR)

    set stmt "insert into $table ($cols) values ($values)"
    trc dbcinsert "statement \"$stmt\""
    set global_options(DBCSTMT) $stmt

    if { [catch {sql exec $dbHandle "$stmt"} dbcErrorMessage] } {
        set code $dbcErrorCode(UNKNOWN_ERROR)
        return $code
    }

    trc dbcinsert "end"

    return $code
}

# ----------------------------------------------------------------------------
# PROC:    dbcupdate
# PARAMS:  table     I   table to update row
#          colValues I   values for cols
#          condition I   string containing the condition (e.g. "WHERE ...")
# RETURN:  0        no error
#         <>0       error-code (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcupdate {table colValues condition} {
    global dbHandle dbcErrorMessage dbcErrorCode

    trc dbcupdate "begin"

    set code $dbcErrorCode(NO_ERROR)

    set stmt "update $table set $colValues $condition"
        set global_options(DBCSTMT) $stmt

    trc dbcupdate "stmt=\"$stmt\""
    if { [catch {sql exec $dbHandle "$stmt"} dbcErrorMessage] } {
        set code $dbcErrorCode(UNKNOWN_ERROR)
        return $code
    }

    trc dbcupdate "end"

    return $code
}

# ----------------------------------------------------------------------------
# PROC:    dbcdelete
# PARAMS:  table     I   table to delete row from
#          condition I   string containing the condition (e.g. "WHERE ...")
# RETURN:  0        no error
#         <>0       error-code (see dbcErrorMessage)
# ----------------------------------------------------------------------------
proc dbcdelete {table condition} {
    global dbHandle dbcErrorMessage dbcErrorCode

    trc dbcupdate "begin"

    set code $dbcErrorCode(NO_ERROR)

    set stmt "delete from $table $condition"
    set global_options(DBCSTMT) $stmt

    trc dbcupdate "stmt=\"$stmt\""
    if { [catch {sql exec $dbHandle "$stmt"} dbcErrorMessage] } {
        set code $dbcErrorCode(UNKNOWN_ERROR)
        return $code
    }

    trc dbcupdate "end"

    return $code

}
