# -----------------------------------------------------------------------------
# FILE:    dbc.tcl
#
# PURPOSE: functionality for database connectivity 
#
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# GLOBAL DATA:
set dbcErrorMessage ""
set dbOpen          "no"

array set dbcErrorCode {
    NO_ERROR            "0"
    MULTIPLE_INDEX      "100"
    UNKNOWN_ERROR	"999"
}

# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
proc trans_tag_DBCOPEN {attr text} {
    global global_options dbOpen dbcErrorMessage

	array set params {
        ERROR		"DBCERROR"
    }
    
	# sql functions
	define_compound_tag "<dbcupdate>content</dbcupdate>"
	define_compound_tag "<dbcinsert>content</dbcinsert>"
	define_compound_tag "<dbcdelete>content</dbcdelete>"
	define_compound_tag "<dbcselect>content</dbcselect>"

	# helpers 
	define_compound_tag "<dbc_column>value</dbc_column>"
	define_compound_tag "<dbc_condition>cond</dbc_condition>"
	define_compound_tag "<dbc_row_content>content</dbc_row_content>"
	define_compound_tag "<dbc_result_begin>content</dbc_result_begin>"
	define_compound_tag "<dbc_result_end>content</dbc_result_end>"
	define_compound_tag "<dbc_error>content</dbc_error>"
	define_compound_tag "<dbc_result>content</dbc_result>"
	define_compound_tag "<dbc_empty_resultset>content</dbc_empty_resultset>"

    trc_register {
        -trans_tag_DBCOPEN
        -trans_tag_DBCCLOSE
		-process_DBCSELECT
		-process_DBCUPDATE
		-process_DBCINSERT
		-process_DBCDELETE
		-process_DBC_COLUMN
		-process_DBC_CONDITION
		-process_DBC_ROW_CONTENT
		-process_DBC_ERROR
		process_DBC_RESULT
        -trans_tag_DBCINSERT
        -dbcopen
        -dbcclose
        -dbcselect
        -dbcfetch
        -dbcupdate
        -dbcinsert
        -dbcdelete
    }
        
    array set params [parse_attrl $attr]
    
    set res ""
    
    # check proper state of database
    if { "$dbOpen" == "yes" } {
        return "$text"
    }
    
    # check for mandatory parameters
    if { "$global_options(DBNAME)" == "" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBOPEN: Database name missing\">"
        return "[parse_html_text $res]$text"
    }
    if { "$global_options(DBUSER)" == "" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCOPEN: Database user missing\">"
        return "[parse_html_text $res]$text"
    }
    
    # open database
    if { ! [dbcopen] } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBOPEN: [rmQuotes $dbcErrorMessage]\">"
        return "[parse_html_text $res]$text"
    }
    trc trans_tag_DBCOPEN "DB opened"
    
    # mark database as opened
    set dbOpen  "yes"
    
    return $res$text
}
# ----------------------------------------------------------------------------


# ----------------------------------------------------------------------------
proc trans_tag_DBCCLOSE {attr text} {
    global global_options dbName dbUser dbOpen dbcErrorMessage

    array set params {
		ERROR		"DBCERROR"
    }

    array set params [parse_attrl $attr]

    set res ""

    # check proper state of database
    if { "$dbOpen" == "no" } {
        return "$text"
    }

    # close database
    if { ! [dbcclose] } {
		append res "<$params(ERROR) 
		MESSAGE=\"DBOPEN: [rmQuotes $dbcErrorMessage]\">"
        return "[parse_html_text $res]$text"
    }

    # mark database as closed
    set dbOpen  "no"

    trc trans_tag_DBCCLOSE "DB closed"

    return $res$text
}
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------

proc process_DBCUPDATE args {
    global dbOpen dbcErrorMessage

	trc process_DBCUPDATE "starting"

	array set params {
		TABLE        ""
		ERROR		"DBCERROR"
		RESULT		"DBCSUCCESS"
	}
	array set params $args
	
	trc process_DBCUPDATE "content: $params(CONTENT)"

	parse_xml_text $params(CONTENT)

    # check proper state of database
    if { "$dbOpen" == "no" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCUPDATE: Database not open\">"
        return "[parse_html_text $res]"
    }

    # check proper state of database
    if { [string length $params(CONDITION)] == 0} {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCUPDATE: Database not open\">"
        return "[parse_html_text $res]"
    }


    foreach col_name [array names cols] {
		trc process_DBCUPDATE "col: $col_name = '$cols($col_name)'"
	}

	set colValues ""
    foreach col_name [array names cols] {
		append colValues ", $col_name = $cols($col_name)"
	}

	# remove leading comma
	set colValues [string range $colValues 2 "end"]

	trc process_DBCUPDATE "values - $colValues"

	# perform database update
	set code [dbcupdate $params(TABLE) $colValues $params(CONDITION)]

	if { $code != 0 } {
		append res "<$params(ERROR) 
		MESSAGE=\"DBCUPDATE: [special2html $dbcErrorMessage]\">"
		return [parse_html_text $res]
	} else {	
		append res "$params(RESULT)"
	}

	# trc trans_tag_DBCUPDATE "end"
	return [parse_html_text $res]
}

# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------

proc process_DBCDELETE args {
    global dbOpen dbcErrorMessage

	trc process_DBCDELETE "starting"

	array set params {
		TABLE           ""
		ERROR		    "DBCERROR"
		RESULT		    "DBCSUCCESS"
	}
	array set params $args
	
    # check proper state of database
    if { "$dbOpen" == "no" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCDELETE: Database not open\">"
        return "[parse_html_text $res]"
    }

	trc process_DBCDELETE "content: $params(CONTENT)"

	parse_xml_text $params(CONTENT)

	# perform database delete
	set code [dbcdelete $params(TABLE) $params(CONDITION)]

	if { $code != 0 } {
		append res "<$params(ERROR) 
		MESSAGE=\"DBCDELETE: [special2html $dbcErrorMessage]\">"
		return [parse_html_text $res]
	} else {	
		append res "$params(RESULT)"
	}

	# trc trans_tag_DBCDELETE "end"
	return [parse_html_text $res]
}

# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
proc process_DBCINSERT args {
    global dbOpen dbcErrorMessage

	trc process_DBCINSERT "starting"

	array set params {
		TABLE        ""
		ERROR		"DBCERROR"
		RESULT		"DBCSUCCESS"
	}
	array set params $args

    # check proper state of database
    if { "$dbOpen" == "no" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCINSERT: Database not open\">"
        return "[parse_html_text $res]"
    }
	
	parse_xml_text $params(CONTENT)

	trc process_DBCINSERT "content processed"


	# generate col=value list
	set colNames ""
	set colValues ""
    foreach col_name [array names cols] {
		append colNames ", $col_name"
		append colValues ", $cols($col_name)"
	}

	# cut ", " from start
	set colNames [string range $colNames 2 end]
	set colValues [string range $colValues 2 end]

	trc process_DBCINSERT "-- $colNames - $colValues"

	# perform database INSERT
	set code [dbcinsert $colNames $params(TABLE) $colValues]

	if { $code != 0 } {
		append res "<$params(ERROR) 
		MESSAGE=\"DBCINSERT: [special2html $dbcErrorMessage]\">"
		return [parse_html_text $res]
	} else {	
		append res "$params(RESULT)"
	}

	# trc trans_tag_DBCINSERT "end"
	return [parse_html_text $res]
}

# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
proc process_DBCSELECT args {
    global global_options dbOpen dbcErrorMessage

	trc process_DBCSELECT "starting"

    array set params {
        COLS            "*"
        TABLES          ""
        CONDITION       ""
		RESULT_BEGIN    ""
		RESULT_END      ""
		ROW_SEPERATOR   ""
		ERROR	 	    "DBCERROR"
		EMPTY_RESULTSET "DBC_EMPTY_RESULTSET"
    }

	array set params $args


    # check proper state of database
    if { "$dbOpen" == "no" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCSELECT: Database not open\">"
        return "[parse_html_text $res]"
    }

	parse_xml_text $params(CONTENT)

	set colPrefixes  {
		"distinct"
	}

    # array set params [parse_attrl $attr]

    set res ""

    # check params
    if { "$params(TABLES)" == "" } {
        append res "<$params(ERROR) 
		MESSAGE=\"DBCSELECT: No tables specified\">"
        return "[parse_html_text $res]"
    }

    # process select statement
    trc trans_tag_DBCSELECT "starting query ..."

    set numRows [dbcselect $params(COLS) $params(TABLES) $params(CONDITION)]

    trc trans_tag_DBCSELECT "... done (result: $numRows rows)"
	
    if { $numRows < 0 } {

		# an error occured !
        append res "<$params(ERROR) MESSAGE=\"DBCSELECT: [rmQuotes $dbcErrorMessage]\">"
        return "[parse_html_text $res]"

    } elseif { $numRows == 0 } {

		# we have an empty result set
		append res "$params(EMPTY_RESULTSET)"
        return "[parse_html_text $res]"

	} 
	# else - we have a result with a least one row ...

    trc trans_tag_DBCSELECT "evaluate query"

	# eleminate column prefixes and split column names
	set clean_cols $params(COLS)
	
	foreach prefix $colPrefixes {
		regsub -all "$prefix" $clean_cols "" clean_cols
	}

	set col_names [split $clean_cols ","]

	# trc trans_tag_DBCSELECT "$col_names"
	# append res "$params(RESULT_BEGIN)"
	append res [parse_html_text $params(RESULT_BEGIN)]

    set row ""
    for {set i 0} {$i < $numRows} {incr i 1} {
        set row [dbcfetch]
        if { [lindex $row 0] < 0 } {
            append res "<$params(ERROR) 
			MESSAGE=\"DBCSELECT: [rmQuotes $dbcErrorMessage]\">"
            return "[parse_html_text $res]"
        } 

        set content [lindex $row 1]
        # trc trans_tag_DBCSELECT "row $i, content: $content"

		set row_text $params(ROW_CONTENT)
		
		set row_text [eval_row_data $row_text [expr "$i + 1"] $col_names $content]
		
		if {$i > 0} { append res [parse_html_text $params(ROW_SEPERATOR) ] }
		append res [parse_html_text "$row_text"]
	}
        # read next (non existing record / row)
        # => make sure that all records are processed and 
        #    SELECT statement is finished 
        #    
        set dummyrow [dbcfetch]        

	# append res "$params(RESULT_END)"

	append res [parse_html_text $params(RESULT_END)]
	
	trc trans_tag_DBCSELECT "end"

	# return [parse_html_text $res]
	return $res
}


# ----------------------------------------------------------------------------


# ----------------------------------------------------------------------------
# sub-tags used by more than one super-tag
# ----------------------------------------------------------------------------

proc process_DBC_COLUMN args {

	trc process_DBC_COLUMN "starting"

	array set params {
		NAME        ""
		TYPE        ""
		ERROR		"DBCERROR"
	}
	array set params $args
	
	# process col value
	trc process_DBC_COLUMN "name: $params(NAME), value: $params(VALUE)"
	set val [parse_xml_text $params(VALUE)]

	# value is not empty
	if {"$val" != ""} {
		# if type is text
		if {[regexp -nocase "text" "$params(TYPE)"]} {
			set val [MaskSQLText $val]
			set val "\'$val\'"
			trc process_DBC_COLUMN $val
		}

		# if type is date
		if {[regexp -nocase "date" "$params(TYPE)"]} {
			set val [MaskSQLDate $val]
			set val "'$val'"
			trc process_DBC_COLUMN $val
		}

		# find precessor tag an set value
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_(DBCUPDATE|DBCINSERT)" "[lindex [info level $i] 0]"]} {
				upvar "#$i" cols($params(NAME)) upval
				set upval $val
				trc process_DBC_COLUMN "val : $upval"
				trc process_DBC_COLUMN "$i - [lindex [info level $i] 0]"
				return ""
			}
		}
	}

	trc process_DBC_COLUMN "end"

	return ""
}

proc process_DBC_CONDITION args {

	array set params {
		COND        ""
		ERROR		"DBCERROR"
	}
	array set params $args
	
	# process condition
	set val [parse_xml_text $params(COND)]
	trc process_DBC_CONDITION "cond: $val"

	# if condition is not empty set in in precessor
	if {"$val" != ""} {
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_(DBCUPDATE|DBCSELECT|DBCDELETE)" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(CONDITION) upval
				set upval $val
				return ""
			}
		}
	}
}

proc process_DBC_ROW_CONTENT args {
	array set params {
		CONTENT     ""
		ERROR		"DBCERROR"
	}
	
	array set params $args

	# get row content
	set val "$params(CONTENT)"
	trc process_DBC_ROW_CONTENT "CONTENT: $val"
	
	#and set it if its not empty
	if {[string length $val] != 0} {
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_DBCSELECT" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(ROW_CONTENT) upval
				set upval $val
				return ""
			}
		}
	}

	return ""
}

proc process_DBC_RESULT_BEGIN args {
	array set params {
		CONTENT     ""
		ERROR		"DBCERROR"
	}
	
	array set params $args

	# get result begin block
	set val "$params(CONTENT)"

	# and set him if he is not empty
	if {[string length $val] != 0} {

		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_DBCSELECT" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(RESULT_BEGIN) upval
				set upval $val
				return ""
			}
		}
	}
	return ""
}

proc process_DBC_RESULT_END args {

	array set params {
		CONTENT     ""
		ERROR		"DBCERROR"
	}
	
	array set params $args

	# get result block 
	set val "$params(CONTENT)"

	# and forget it :o)
	if {[string length $val] != 0} {

		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_DBCSELECT" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(RESULT_END) upval
				set upval $val
				return ""
			}
		}
	}
	return ""
}

proc process_DBC_ERROR args {

	array set params {
		CONTENT        "DBCERROR"
	}
	array set params $args
	
	# process condition
	set val $params(CONTENT)
	trc process_DBC_ERROR "content: $val"

	# if error is not empty set in in precessor
	if {"$val" != ""} {
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_(DBCINSERT|DBCUPDATE|DBCSELECT|DBCDELETE)" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(ERROR) upval
				set upval $val
				return ""
			}
		}
	}
}

proc process_DBC_RESULT args {

	array set params {
		CONTENT        "DBCSUCCESS"
	}
	array set params $args
	
	# process condition
	set val $params(CONTENT)
	trc process_DBC_RESULT "content: $val"

	# if error is not empty set in in precessor
	if {"$val" != ""} {
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_(DBCINSERT|DBCUPDATE|DBCDELETE)" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(RESULT) upval
				set upval $val
				return ""
			}
		}
	}
}

proc process_DBC_EMPTY_RESULTSET args {

	array set params {
		CONTENT        "DBC_EMPTY_RESULTSET"
	}
	array set params $args
	
	# process condition
	set val $params(CONTENT)
	trc process_DBC_EMPTY_RESULTSET "content: $val"

	# if error is not empty set in in precessor
	if {"$val" != ""} {
		for {set i [info level]} {$i > 0} {incr i -1} {			
			if {[regexp -nocase "process_(DBCSELECT)" "[lindex [info level $i] 0]"]} {
				upvar "#$i" params(EMPTY_RESULTSET) upval
				set upval $val
				return ""
			}
		}
	}
}




# ----------------------------------------------------------------------------
# additional procs
# ----------------------------------------------------------------------------
proc MaskSQLText { str } {
	regsub -all \' $str "\'\'" str
	
	return $str
}

proc MaskSQLDate { str } {
	trc process_DBC_COLUMN "--> $str"

	set date [clock scan $str]
	
	return [clock format $date -format "%Y-%b-%d"]
}


proc eval_row_data {__row_text __row_num __col_names __content} {
    global global_options
	for {set __j 0} {$__j < [llength $__content]} {incr __j 1} {			
		set __col_name [string trim [lindex $__col_names $__j]]
		set __col_num [expr "$__j + 1"]
		set __col_data [special2html [lindex $__content $__j]]
		
		set __command "set COLUMN($__col_name) {$__col_data}; set COLUMN($__col_num) {$__col_data}"

		# trc trans_tag_DBCSELEC "$__command"
		eval $__command
	}
	set ROW_NUM $__row_num
	set __row_text [subst -nobackslashes -nocommands $__row_text]

	return $__row_text

}

proc rmQuotes { str } {
	set res $str
	regsub -all {\"|\'} $str {} res

	return [string trim $res]
}
