
namespace eval Util {
	# returns a timestamp formatted "yyyy-mm-dd:hh:mm:ss"
	namespace export get_Timestamp

	# returns a formid, unique identifier for a questionary
	namespace export generate_FormID

	# returns a sessionid, unique identifier of a user
	namespace export generate_SessionID

	# returns the ip-address of the remote user
	namespace export get_Remote_Address

	# checks the formid and password, and return an error-message if they are incorrect
	# Parameters:	formid: the unique identifier of a questionary to be checked
	#					password: the password unlocking the questionary specified by formid
	namespace export check_FormID_And_Password

	# checks the orderno provided, and return an error-message if it is incorrect
	# Parameters:	formid: unique identifier for a questionary
	#					orderno: unique identifier of a form-element
	#					method: 	hastoexist: the orderno must exist already
	#								| strict: orderno must be a number, must not be empty nor contain any wildcards
	#								| nice: any string is ok for the orderno, if there are no visible elements in the forms
	namespace export check_OrderNo

	# checks a number, and return an error-message if it is incorrect
	# Parameters:	number: the number to be checked, an integer
	#					lowerborder: number must be equal or bigger than this value
	#					upperborder: number must be equal or less than this value
	namespace export check_Number

	# checks a color, and return an error-message if it is incorrect
	# Parameters:	color: string to be checked
	namespace export check_Color

	# checks an url, and return an error-message if it is incorrect
	# Parameters:	url: string to be checked
	namespace export check_URL

	# checks a boolean, and return an error-message if it is incorrect
	# Parameters:	boolean: string to be checked
	namespace export check_Boolean

	# checks if a list contains an item, optionally recursively in a sublist
	# Parameters:	itemlist: any list, eventually containing sublists
	#					item: the item to be searched for in itemlist
	#					indexname: the name of the index-variable in the calling proc
	#					keycolumnlist: if itemlist contains sublists, the indexes
	#					mode
	namespace export is_In_List

	# deletes an item in a list
	# Parameters: 	
	namespace export delete_From_List
	# extracts a list out of a list, identified by a key value
	namespace export get_Sub_List
	# determines the maximum list length from a list of lists
	namespace export get_Max_List_Length
	# returns if color is correctly formatted
	namespace export wrong_Color_Format
	# returns if a string could be evaluated as a boolean
	namespace export is_True
	# abbreviate a string
	namespace export abbreviate
	# determines the kind of an element by its describing attributes
	namespace export determine_Element_Type
	# converts the element type into a german string
	namespace export element_Type_To_String
	# converts the spacer type into a german string
	namespace export spacer_Type_To_String
}


########################################################################
###########################    Oeffentlich    ##########################
########################################################################


proc Util::get_Timestamp {} {
	return [clock format [clock seconds] -format "%Y-%m-%d:%H:%M:%S" -gmt 1]
}


########################################################################


proc Util::generate_FormID {} {
	return [get_Timestamp]
}


########################################################################


proc Util::generate_SessionID {} {
	set firstpart [get_Timestamp]
	set lastpart [clock clicks -milliseconds]
	return "$firstpart.$lastpart"
}


########################################################################


proc Util::get_Remote_Address {} {
	# ip-address must be extracted from global env-array
	global env
	set userid ""
	set result ""
	
	# either get remote-address
	if	{	([catch {set userid "$env(REMOTE_ADDR)"}])
		||	([string length $userid] == 0)
	}	{
		# or get remote host
		if	{	([catch {set userid "$env(REMOTE_HOST)"}])
			||	([string length $userid] == 0)
		}	{
			# else return error-message
			set result "no id provided"
		} else {
			set result $userid 
		}
	} else {
		set result $userid
	}
	
	return "$result"
}


########################################################################


proc Util::check_FormID_And_Password	{ 	{formid {}}
														{password {}}
													}	{
	# formid must have a length > 0												
	if {[string length "$formid"] < 1} {
		return [Error::display_Msg {
			Es wurde kein Fragebogen gew&auml;hlt. Bitte w&auml;hlen Sie auf 
			der Hauptseite einen Fragebogen.
		}]
	} 
	
	# formid must not contain following chars: "%"
	if {[regexp {%} "$formid"]} {
		return [Error::display_Msg {
			Die Fragebogen-ID enth&auml;lt das verbotene Zeichen 
			&quot;%&quot;. Der Fragebogen kann daher nicht ge&ouml;ffnet
			werden.
		}]
	}
	
	# formid must exist
	if {! [DBOperations::form_Exists "$formid"]} {
		return [Error::invalid_Form_Selected]
	}

	# password must match the password stored for formid
	if	{[DBOperations::password_Matches "$formid" "$password"]} {
		return ""
	} else {
		return [Error::wrong_Password]
	}	
}		


########################################################################


proc Util::check_OrderNo	{ 	{formid {}}
										{orderno {}}
										{method "strict"}
									}	{
	# if lax checking required only and no visible elements in questionary
	if	{	([string equal "nice" "$method"])
		&&	([DBOperations::no_Visible_Elements "$formid"])
	}	{
		# then any orderno is ok, we won't need it
		return ""
	}
	
	# orderno must have a length > 0
	if {[string length "$orderno"] < 1} {
		return [Error::display_Element_Msg {
			Es wurde kein Fragebogenelement gew&auml;hlt. Bitte w&auml;hlen 
			Sie auf der Seite zum Bearbeiten der Frageb&ouml;gen ein Element.
		}]
	} 
	
	# orderno must be an integer. 
	if {! [string is integer -strict "$orderno"]} {
		return [Error::display_Element_Msg {
			Die von Ihnen gew&auml;hlte Positionsnummer ist keine Zahl und
			daher ung&uuml;ltig. Bitte geben Sie f&uuml;r die 
			Positionsnummer eine ganze Zahl an.
		}]
	}
	
	# if orderno has to exist, as we want to update an element in a questionary,
	# this is signaled by the "hastoexist"-method
	if {	([string equal "hastoexist" "$method"])
		&& (! [DBOperations::orderno_Exists "$formid" "$orderno"])
	}	{
		return [Error::display_Element_Msg {
			Das von Ihnen gew&auml;hlte Fragebogenelement existiert nicht
			mehr oder ist ungltig. Bitte w&auml;hlen Sie auf der Seite zum
			Bearbeiten der Frageb&ouml;gen ein g&uuml;ltiges Element.
		}]
	}		
	
	# everything ok
	return ""
}


########################################################################


proc Util::check_Number	{	{number ""}
									{lowerborder ""}
									{upperborder ""}
								}	{
	# number must be an integer, an may not be empty
	if {[string is integer -strict $number]} {
		# if lowerborder was provided, check number against it
		if {[string is integer -strict $lowerborder]} {
			# number must be equal or bigger than lowerborder
			if {$number < $lowerborder} {
				return [subst {Es wurde eine Zahl grer $lowerborder 
					erwartet, statt dessen haben Sie "$number" eingegeben.
				}]
			}
		}
		# if upperborder was provided, check number against it
		if {[string is integer -strict $upperborder]} {
			# number must be equal or smaller than upperborder
			if {$number > $upperborder} {
				return [subst {Es wurde eine Zahl kleiner $upperborder 
					erwartet, statt dessen haben Sie "$number" eingegeben.
				}]
			}
		}
		# everything ok. 
		return ""
	} else {
		# number is not an integer
		return [subst {Es wurde eine Zahl erwartet, statt dessen haben Sie
			"$number" eingegeben.
		}]
	}
}


########################################################################


proc Util::check_Color { {color ""} } {
	# check the color-format
	if {[wrong_Color_Format $color]} {
		# the format is incorrect
		return [subst {
			Das folgende ist keine gltige Farbangabe: "$color".
		}]
	} else {
		# everything ok
		return ""
	}
}


########################################################################


proc Util::check_URL { {url ""} } {
	# very lax check of an url, to get relative as well as absolute urls
	if {[regexp {((.+?)://)?(.+?)(?::([0-9]+))?(/.*)?} $url]} {
		# everything ok
		return ""
	} else {
		# input seems not to be an url
		return [subst {
			Das folgende ist keine URL: "$url".
		}]
	}
}


########################################################################


proc Util::check_Boolean { {boolean ""} } {
	# these are the values that can be evaluated as booleans
	set booleanvalues [list ja nein wahr falsch true false yes no]
	# make the input-string comparable
	set lowercasecomparestr [string tolower $boolean]
	# input must not be empty, and must any of the values specified in 
	# booleanvalues or a boolean expression like eg. (a >= b)
	if	{	([string length $boolean] > 0)
		&&	(	([is_In_List $booleanvalues "$lowercasecomparestr"])
			|| (! [catch {expr "$boolean"}])
			)
	}	{
		# everything ok
		return ""
	} else {
		# input could not be evaluated as a boolean
		return [subst {
			Das folgende ist kein boolescher Wert: "$boolean".
		}]
	}
}


########################################################################


proc Util::is_In_List	{	{itemlist {}}
									{item ""}
									{indexname ""}
									{keycolumnlist {0}}
									{mode "exact"}
								}	{
	# if item or itemlist is empty, do not search
	if {	([string length "$item"] == 0)
		||	([llength $itemlist] < 1)
	} {
		# no element found
		return 0
	} else {
		# item and itemlist were specified, let's go!
		# if index was specified, access index-variable in calling proc, 
		# and intialize it with invalid value
		if {[string length $indexname] > 0} {
			upvar 1 "$indexname" index
			set index -1
		}
		# if itemlist contain sublists and a list of key-columns was 
		# specified, search for element recursively, indexing the itemlist
		# by the element specified in keycolumnlist at the first position
		if {[get_Max_List_Length $itemlist] > 1} {
			# if no keycolumnlist specified
			if {[llength $keycolumnlist] < 1} {
				# use first element of sublists per default as a key
				set keycolumn 0
			} else {
				# use specified element of sublists as a key
				set keycolumn [lindex $keycolumnlist 0]
			}
			# extract the key element from each of item of itemlist
			foreach element $itemlist {
				# build new list containing only the key elements
				lappend recursivelist [lindex $element $keycolumn]
			}
			# recurse: check list containing the key-elements for the 
			# existence of the requested item 
			return [is_In_List $recursivelist "$item" "index" [lrange $keycolumnlist 1 end] "$mode"]
		} else {
			# no sublists contained in list - end of recursion
			# search itemlist for item and memorize the index 
			set index [lsearch -$mode $itemlist "$item"]
			# result: 1 if the item was found, 0 else
			return [expr "! ($index == -1)"]
		}
	}
}


########################################################################


proc Util::delete_From_List	{	{itemlistname {}}
											{item ""} 
										}	{
	# if no name of a variable provided, start with an empty list
	if {[string length "$itemlistname"] < 1} {
		set itemlist {}
	} else {
		upvar 1 "$itemlistname" itemlist
		if {! [info exists itemlist]} {
			set itemlist {}
		}
	}
	
	# check list and item provided
	if {	([string length "$item"] > 0)
		&&	([llength $itemlist] > 0)
		&&	([is_In_List $itemlist $item index])
	}	{
		set result 0
		# if item still is found in the list
		while {! ($index == -1)} {
			if {[llength $itemlist] < 2} {
				# ok, the list has less than 2 items and we still found the
				# one we're looking for - delete the whole list, and exit
				# this damn loop fast
				set itemlist {}
				set index -1
			} else {
				# delete the item
				set itemlist [lreplace $itemlist $index $index]
				# look for the next occurance - the result is "insignificant",
				# as we borgs say, we just need the index
				is_In_List $itemlist $item index
			}
			# count how many elements have been assimilated - eh, deleted
			incr result
		}
		# return number of deleted elements
		return $result
	} else {
		# if list is empty, or no item provided, or the item is not in 
		# the list, we have deleted 0 elements :-(
		return 0
	}	
}
										

########################################################################


proc Util::get_Sub_List	{	{itemlist {}}
									{keyvalue {}}
								}	{
	set result {}
	# from a list of list, extract the sublists matching the key
	foreach item $itemlist {
		set currentkey [lindex $item 0]
		if {[string equal "$keyvalue" "$currentkey"]} {
			lappend result [lindex $item 1]
		}
	} 
	return $result
}


########################################################################


proc Util::get_Max_List_Length { lists } {
	set maxlength 0
	# get the  maximum length of all sublists
	foreach arg $lists {
		set currentlength [llength $arg]
		if {$currentlength > $maxlength} {
			set maxlength $currentlength
		}
	}
	return $maxlength
}


########################################################################


proc Util::wrong_Color_Format { {color {}} } {
	if {[string length "$color"] == 0} {
		return 0
	} 
	 
	set validcolornames {aqua black blue cyan fuchsia gray green lime 
								magenta maroon navy olive purple red silver 
								teal white yellow}
	if {[is_In_List $validcolornames "[string tolower $color]"]} {
		return 0
	}
	
	set numericcolorregexp {^#[[:xdigit:]]{6}$}
	return [expr ! [regexp -nocase "$numericcolorregexp" "$color"]]
}


########################################################################


proc Util::is_True { {str ""} } {
	# check if str might be an expression which could be evaluated
	if {[catch {set expression [expr "$str"]}]} {
		# would str be evaluated, it would be false
		set expression 0
	}
	set lowercasecomparestr [string tolower $str]
	return [expr {
			([string equal "$lowercasecomparestr" "ja"])
		||	([string equal "$lowercasecomparestr" "true"])
		||	([string equal "$lowercasecomparestr" "yes"])
		|| ([string equal "$lowercasecomparestr" "wahr"])
		||	([expr $expression])
	}]
}


########################################################################


proc Util::abbreviate	{	{str ""}
									{tolength 1}
									{end "."}
									{wholewords 0}
								}	{
	# check if str needs to be abbreviated
	if {[string length "$str"] <= $tolength} {
		# no need to abbreviate string
		return [special2html "$str"]
	} else {
		set len [expr $tolength - 1]
		# if we want no interrupted words
		if {$wholewords} {
			# break after a space
			set result [string range "$str" 0 [expr "[string wordstart $str $len] - 1"]]
		} else {
			# break anywhere
			set result [string range "$str" 0 $len]
		}
		return [special2html "$result$end"]
	}
}


########################################################################


proc Util::determine_Element_Type {	{describinglist {}} } {
	# decide by some significant attributes which type an element is of.
	# WARNING, PROGRAMMER: if these elements are missing, we will maybe
	# make the wrong decisions. and this can might lead to the point,
	# where our precious space shuttle just... i don't want to think 
	# about it. but we have no choice
	if {[Util::is_In_List $describinglist "QST_SHOW_NUMBER"]} {
		# element is some kind of question
		if {[Util::is_In_List $describinglist "INP_SEL_SUB_QST_ALIGNMENT"]} {
			# element is a block question
			return "QST_BLOCK"
		} elseif {[Util::is_In_List $describinglist "INP_TXT_WIDTH"]} {
			# element is a text question
			return "QST_TEXT"
		} elseif {[Util::is_In_List $describinglist "INP_SEL_TYPE"]} {
			# element is a select question
			return "QST_SELECT"			
		} else {
			# we don't know this kind of question, but won't guess blindly
			# anyway to keep this thing debugable
			return "QST_UNKNOWN"
		}
	} elseif {[Util::is_In_List $describinglist "TXT_ALIGNMENT"]} {
		# element is a text block
		return "TEXT"
	} elseif {[Util::is_In_List $describinglist "SEP_TYPE"]} {
		# element is a seperator
		return "SEPERATOR"
	} else {
		# we don't know this kind of element at all, but won't guess 
		# blindly anyway to keep this thing debugable (have i said that
		# before already?)
		return "UNKNOWN"
	}
}


########################################################################


proc Util::element_Type_To_String { {elementtype ""} } {
	# isn't that difficult to understand, is it?
	switch -exact -- $elementtype {
		QST_SELECT {return "Auswahlfrage"}
		QST_TEXT {return "Textfrage"}
		QST_BLOCK {return "Blockfrage"}
		QST_UNKNOWN {return "{unbekannter Fragetyp}"}
		TEXT {return "Text"}
		SEPERATOR {return "Trenner"}
		UNKNOWN {return "{unbekannter Elementtyp}"}
		default {return "{unbekannter Elementtyp}"}
	}
}


########################################################################


proc Util::spacer_Type_To_String { {spacertype ""} } {
	set spacertype [string tolower "$spacertype"]
	switch -exact -- $spacertype {
		space {return "Zwischenraum"}
		hr {return "Balken"}
		img {return "Bild"}
		default {return "{unbekannter Typ}"}
	}
}


